# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutron_lib.utils import helpers

from networking_l2gw.l2gatewayclient.l2gw_client_ext import _l2_gateway
from neutronclient.common import extension

from midonet.neutron._i18n import _


def add_known_arguments(self, parser):
    parser.add_argument(
        '--device',
        metavar='device_id=DEVICE_ID,segmentaion_id=SEGMENTAION_ID',
        action='append', dest='devices', type=helpers.str2dict,
        help=_('Device id and segmentation id of l2gateway. '
               '--device option can be repeated'))


def args2body(self, parsed_args):
        if parsed_args.devices:
            devices = parsed_args.devices
        else:
            devices = []
        body = {'l2_gateway': {'devices': devices}}
        if parsed_args.name:
            l2gw_name = parsed_args.name
            body['l2_gateway']['name'] = l2gw_name
        return body


class L2Gateway(_l2_gateway.L2Gateway):
    """L2Gateway resource class

    This class loads parameters for L2Gateway.
    """

    pass


class L2GatewayCreate(extension.ClientExtensionCreate, L2Gateway):
    """Create l2gateway information for midonet."""

    shell_command = 'midonet-l2-gateway-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='GATEWAY-NAME',
            help=_('Descriptive name for logical gateway.'))
        add_known_arguments(self, parser)

    def args2body(self, parsed_args):
        body = args2body(self, parsed_args)
        if parsed_args.tenant_id:
            body['l2_gateway']['tenant_id'] = parsed_args.tenant_id
        return body



# Copyright 2015 Midokura SARL
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import provider_net as pnet
from neutron_lib.api import validators
from neutron_lib.db import model_base
from neutron_lib import exceptions as n_exc
from oslo_log import log as logging
import sqlalchemy as sa
from sqlalchemy import orm

from neutron.db import models_v2

from midonet.neutron._i18n import _
from midonet.neutron.common import constants as m_const


LOG = logging.getLogger(__name__)


class NetworkBinding(model_base.BASEV2):
    __tablename__ = 'midonet_network_bindings'

    network_id = sa.Column(sa.String(36), sa.ForeignKey('networks.id'),
                           primary_key=True)
    network_type = sa.Column(sa.String(length=255), nullable=False)
    network = orm.relationship(models_v2.Network,
                               backref=orm.backref("network_binding",
                                                   lazy='joined',
                                                   uselist=False,
                                                   cascade='delete'))


class MidonetProviderNetworkMixin(object):

    def _get_net_type(self, session, network_id):
        net_binding = session.query(NetworkBinding).filter_by(
            network_id=network_id).first()
        if net_binding:
            return net_binding.network_type
        return m_const.TYPE_MIDONET

    def _extend_provider_network_dict(self, context, network):
        id = network['id']
        network[pnet.NETWORK_TYPE] = self._get_net_type(context.session, id)

    def _process_provider_create(self, network):

        net_type = network.get(pnet.NETWORK_TYPE)
        if not validators.is_attr_set(net_type):
            return None

        if net_type == m_const.TYPE_MIDONET:
            return None

        if net_type != m_const.TYPE_UPLINK:
            msg = _('Unsupported network type %(type)s detected '
                    'in a create network request.') % {'type': net_type}
            raise n_exc.InvalidInput(error_message=msg)

        return net_type

    def _create_provider_network(self, context, network):
        net_type = self._process_provider_create(network)
        if net_type:
            with context.session.begin(subtransactions=True):
                context.session.add(NetworkBinding(network_id=network['id'],
                                                   network_type=net_type))

    def _match_attrs(self, net, filters):
        return (not filters.get(pnet.NETWORK_TYPE) or
                net.get(pnet.NETWORK_TYPE) in filters[pnet.NETWORK_TYPE])

    def _provider_network_matches_filters(self, network, filters):
        if not filters:
            return True

        if validators.is_attr_set(network.get(pnet.NETWORK_TYPE)):
            return self._match_attrs(network, filters)
        else:
            return True

    def _filter_nets_provider(self, networks, filters):
        return [network
                for network in networks
                if self._provider_network_matches_filters(network, filters)
                ]



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

import sqlalchemy as sa

from neutron_lib.db import model_base


DATA_VERSIONS_TABLE = 'midonet_data_versions'

STARTED = "STARTED"
COMPLETED = "COMPLETED"
ERROR = "ERROR"
ABORTED = "ABORTED"


class DataVersion(model_base.BASEV2):
    __tablename__ = DATA_VERSIONS_TABLE
    id = sa.Column(sa.Integer(), primary_key=True)
    sync_started_at = sa.Column(sa.DateTime())
    sync_finished_at = sa.Column(sa.DateTime())
    sync_status = sa.Column(sa.String(length=50))
    sync_tasks_status = sa.Column(sa.String(length=50))
    stale = sa.Column(sa.Boolean(), nullable=False)


def get_last_version(session):
    data_versions = session.query(DataVersion)
    return data_versions.order_by(DataVersion.id.desc()).first()


def update_last_version_status(session, status):
    dv = get_last_version(session)
    dv.update({'sync_tasks_status': status})


def complete_last_version(session):
    update_last_version_status(session, COMPLETED)


def error_last_version(session):
    update_last_version_status(session, ERROR)


def abort_last_version(session):
    update_last_version_status(session, ABORTED)


def get_last_version_id(session):
    data_versions = session.query(DataVersion)
    dv = data_versions.order_by(DataVersion.id.desc()).first()
    if dv is None:
        return None
    else:
        return dv.id


def get_data_version_states(session):
    dv = session.query(DataVersion).order_by(DataVersion.id.desc()).first()
    if dv is None:
        return None, None
    else:
        return dv.sync_status, dv.sync_tasks_status


def get_data_versions(session):
    return session.query(DataVersion).all()


def create_data_version(session):
    data_version = DataVersion(sync_started_at=datetime.datetime.utcnow(),
                               sync_tasks_status=STARTED,
                               stale=False)
    session.add(data_version)



# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from oslo_serialization import jsonutils

from neutronclient.common import extension
from neutronclient.neutron import v2_0 as gw_deviceV20

from midonet.neutron._i18n import _


def add_name_and_tunnel_ips_to_arguments(parser):
    parser.add_argument(
        '--name', dest='name',
        help=_('User defined device name.'))
    parser.add_argument(
        '--tunnel-ip', metavar='TUNNEL_IP',
        action='append', dest='tunnel_ips',
        help=_('IP address on which gateway device originates or '
               'terminates tunnel.'))


def _format_remote_mac_entries(gw_device):
    try:
        return '\n'.join([jsonutils.dumps(rm_entry) for rm_entry
                          in gw_device['remote_mac_entries']])
    except (TypeError, KeyError):
        return ''


class GatewayDevice(extension.NeutronClientExtension):
    resource = 'gateway_device'
    resource_plural = 'gateway_devices'
    path = 'gw/gateway_devices'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


class GatewayDeviceCreate(extension.ClientExtensionCreate, GatewayDevice):
    """Create Gateway Device information."""

    shell_command = 'gateway-device-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--management-ip',
            dest='management_ip',
            help=_('Management IP to the device. Defaults to None.'))
        parser.add_argument(
            '--management-port',
            dest='management_port',
            help=_('Management port to the device. Defaults to None.'))
        parser.add_argument(
            '--management-protocol',
            dest='management_protocol',
            help=_('Management protocol to manage the device: ovsdb or none. '
                   'If management ip and port are specified, '
                   'defaults to ovsdb. Otherwise to none.'))
        parser.add_argument(
            '--type',
            metavar='<hw_vtep | router_vtep | network_vlan>',
            choices=['hw_vtep', 'router_vtep', 'network_vlan'],
            help=_('Type of the device: hw_vtep, router_vtep or network_vlan. '
                   'Defaults to hw_vtep'))
        parser.add_argument(
            '--resource-id',
            dest='resource_id',
            help=_('Resource UUID or None (for type router_vtep will '
                   'be router UUID and for type network_vlan will be network '
                   'UUID)'))
        add_name_and_tunnel_ips_to_arguments(parser)

        return parser

    def args2body(self, args):
        body = {}
        attributes = ['name', 'type', 'management_ip',
                      'management_port', 'management_protocol',
                      'resource_id', 'tenant_id', 'tunnel_ips']
        gw_deviceV20.update_dict(args, body, attributes)

        return {'gateway_device': body}


class GatewayDeviceList(extension.ClientExtensionList, GatewayDevice):
    """List Gateway Devices."""

    shell_command = 'gateway-device-list'
    list_columns = ['id', 'name', 'type', 'resource_id', 'tunnel_ips']
    pagination_support = True
    sorting_support = True

    _formatters = {'remote_mac_entries': _format_remote_mac_entries, }


class GatewayDeviceShow(extension.ClientExtensionShow, GatewayDevice):
    """Show information of a given gateway-device."""

    shell_command = 'gateway-device-show'


class GatewayDeviceDelete(extension.ClientExtensionDelete, GatewayDevice):
    """Delete a given gateway-device."""

    shell_command = 'gateway-device-delete'


class GatewayDeviceUpdate(extension.ClientExtensionUpdate, GatewayDevice):
    """Update a given gateway-device."""

    shell_command = 'gateway-device-update'

    def add_known_arguments(self, parser):
        add_name_and_tunnel_ips_to_arguments(parser)

    def args2body(self, args):
        body = {}
        attributes = ['name', 'tunnel_ips']
        gw_deviceV20.update_dict(args, body, attributes)

        return {'gateway_device': body}



# Copyright (C) 2016 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from neutron.tests import base

from midonet.neutron.services.qos import driver as qos_driver


class QoSDriverTestCase(base.BaseTestCase):
    def setUp(self):
        super(QoSDriverTestCase, self).setUp()
        self._mock_client = mock.Mock()
        with mock.patch("midonet.neutron.client.base.load_client",
                        return_value=self._mock_client), \
            mock.patch(
                "neutron_lib.services.qos.base.DriverBase.__init__"):
            self._driver = qos_driver.MidoNetQosDriver()

    def test_qos_policy_create(self):
        mock_context = mock.Mock()
        mock_policy_obj = mock.Mock()
        self._driver.create_policy(mock_context, mock_policy_obj)
        self.assertEqual([
            mock.call.create_qos_policy(mock_context, mock_policy_obj),
        ], self._mock_client.mock_calls)

    def test_qos_policy_update(self):
        mock_context = mock.Mock()
        mock_policy_obj = mock.Mock()
        self._driver.update_policy(mock_context, mock_policy_obj)
        self.assertEqual([
            mock.call.update_qos_policy(mock_context, mock_policy_obj),
        ], self._mock_client.mock_calls)

    def test_qos_policy_delete(self):
        mock_context = mock.Mock()
        mock_policy_obj = mock.Mock()
        self._driver.delete_policy(mock_context, mock_policy_obj)
        self.assertEqual([
            mock.call.delete_qos_policy(mock_context, mock_policy_obj),
        ], self._mock_client.mock_calls)



# Copyright 2014 Midokura SARL
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from alembic import config as alembic_config
from oslo_config import cfg
from oslo_serialization import jsonutils
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from neutron.db.migration import cli as n_cli

from midonet.neutron._i18n import _
import midonet.neutron.db.data_state_db as ds_db
import midonet.neutron.db.data_version_db as dv_db
from midonet.neutron.db import task_db


CONF = n_cli.CONF


def get_session(config):
    connection = config.neutron_config.database.connection
    engine = create_engine(connection)
    Session = sessionmaker(bind=engine)
    return Session()


def task_list(config, cmd):
    """Lists all of the tasks in the task table.

    Optionally filters tasks to show unprocessed tasks.

    :param config: contains neutron configuration, like database connection.
    :param cmd: unused, but needed in the function signature by the command
        parser.
    """
    session = get_session(config)
    printer = config.print_stdout
    line = "%-7s%-11s%-20s%-40s%-20s"
    printer(line, "id", "type", "data type", "resource id", "time")
    printer(line, "--", "----", "---------", "-----------", "----")

    def print_task(task):
        printer(line, task.id, task.type, task.data_type, task.resource_id,
                task.created_at)
    show_unprocessed = config.neutron_config.command.u
    tasks = task_db.get_task_list(session, show_unprocessed)
    [print_task(task) for task in tasks]


def task_clean(config, cmd):
    """Removes all of the tasks that have been processed by the cluster.

    Remove those tasks from the task table.

    :param config: contains neutron configuration, like database connection.
    :param cmd: unused, but needed in the function signature by the command
        parser.
    """
    session = get_session(config)
    task_db.task_clean(session)


def task_resource(config, cmd):
    """Lists all of the resources represented in the task table.

    This will only show the most updated information, and it will not
    consider resources that have been removed from the task table.

    :param config: contains neutron configuration, like database connection.
    :param cmd: unused, but needed in the function signature by the command
        parser.
    """
    session = get_session(config)
    printer = config.print_stdout
    data = task_db.get_current_task_data(session)
    for data_type in data:
        printer(data_type + "S: \n")
        for res in data[data_type]:
            data_json = jsonutils.loads(data[data_type][res])
            printer(jsonutils.dumps(data_json, indent=4, sort_keys=True))


def data_show(config, cmd):
    """Dumps the contents of the data state table.

    :param config: contains neutron configuration, like database connection.
    :param cmd: unused, but needed in the function signature by the command
        parser.
    """
    session = get_session(config)
    printer = config.print_stdout
    data_state = ds_db.get_data_state(session)
    line = "%-25s : %s"
    printer(line, "last processed task id", data_state.last_processed_task_id)
    printer(line, "last updated", data_state.updated_at)
    printer(line, "active version", data_state.active_version)
    readonly = "True" if data_state.readonly else "False"
    printer(line, "read only", readonly)


def data_readonly(config, cmd):
    """Sets the task table access state to "read only"

    :param config: contains neutron configuration, like database connection.
    :param cmd: unused, but needed in the function signature by the command
        parser.
    """
    session = get_session(config)
    ds_db.set_readonly(session)


def data_readwrite(config, cmd):
    """Sets the task table access state to "read and write"

    :param config: contains neutron configuration, like database connection.
    :param cmd: unused, but needed in the function signature by the command
        parser.
    """
    session = get_session(config)
    ds_db.set_readwrite(session)


def data_version_list(config, cmd):
    """Lists the statuses of the versions in the midonet data version table.

    :param config: contains neutron configuration, like database connection.
    :param cmd: unused, but needed in the function signature by the command
        parser.
    """
    session = get_session(config)
    data_versions = dv_db.get_data_versions(session)
    printer = config.print_stdout
    line = "%-7s%-20s%-20s%-10s"
    printer(line, "id", "sync_status", "sync_tasks_status", "stale")
    printer(line, "--", "-----------", "-----------------", "-----")
    for dv in data_versions:
        printer(line, dv.id, dv.sync_status, dv.sync_tasks_status, dv.stale)


def data_version_sync(config, cmd):
    pass


def data_version_activate(config, cmd):
    pass


def add_command_parsers(subparsers):
    parser = subparsers.add_parser('task-list')
    parser.add_argument('-u', action='store_true')
    parser.set_defaults(func=task_list)
    parser = subparsers.add_parser('task-clean')
    parser.set_defaults(func=task_clean)
    parser = subparsers.add_parser('task-resource')
    parser.set_defaults(func=task_resource)
    parser = subparsers.add_parser('data-show')
    parser.set_defaults(func=data_show)
    parser = subparsers.add_parser('data-readonly')
    parser.set_defaults(func=data_readonly)
    parser = subparsers.add_parser('data-readwrite')
    parser.set_defaults(func=data_readwrite)
    parser = subparsers.add_parser('data-version-list')
    parser.set_defaults(func=data_version_list)
    parser = subparsers.add_parser('data-version-sync')
    parser.set_defaults(func=data_version_sync)
    parser = subparsers.add_parser('data-version-activate')
    parser.set_defaults(func=data_version_activate)


command_opt = cfg.SubCommandOpt('command',
                                title='Command',
                                help=_('Available commands'),
                                handler=add_command_parsers)

# Override the db management options with our own version
CONF.unregister_opt(n_cli.command_opt)
CONF.register_cli_opt(command_opt)


def get_alembic_config():
    config = alembic_config.Config(os.path.join(os.path.dirname(__file__),
                                                'alembic.ini'))
    config.set_main_option('script_location',
                           'midonet.neutron.db.migration:alembic_migration')
    return config


def main():
    CONF(project='neutron')
    config = get_alembic_config()
    # attach the Neutron conf to the Alembic conf
    config.neutron_config = CONF

    CONF.command.func(config, CONF.command.name)



# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutronclient.common import extension
from neutronclient.neutron import v2_0 as gw_deviceV20

from midonet.neutron._i18n import _


def _get_gateway_device_id(client, gw_device_id_or_name):
    return gw_deviceV20.find_resourceid_by_name_or_id(client, 'gateway_device',
                                                      gw_device_id_or_name)


class RemoteMacEntry(extension.NeutronClientExtension):
    parent_resource = 'gateway_devices'
    resource = 'remote_mac_entry'
    resource_plural = 'remote_mac_entries'
    object_path = '/gw/%s/%%s/%s' % (parent_resource, resource_plural)
    resource_path = '/gw/%s/%%s/%s/%%%%s' % (parent_resource, resource_plural)
    versions = ['2.0']

    def add_known_arguments(self, parser):
        super(RemoteMacEntry, self).add_known_arguments(parser)
        parser.add_argument(
            'gateway_device', metavar='GATEWAY_DEVICE',
            help=_('ID of the gateway device.'))

    def set_extra_attrs(self, parsed_args):
        self.parent_id = _get_gateway_device_id(self.get_client(),
                                                parsed_args.gateway_device)


class RemoteMacEntryCreate(extension.ClientExtensionCreate, RemoteMacEntry):
    """Create Gateway Device Remote Mac Entry information."""

    shell_command = 'gateway-device-remote-mac-entry-create'

    def add_known_arguments(self, parser):
        super(RemoteMacEntryCreate, self).add_known_arguments(parser)
        parser.add_argument(
            '--mac-address', dest='mac_address',
            required=True,
            help=_('Remote MAC address'))
        parser.add_argument(
            '--vtep-address', dest='vtep_address',
            required=True,
            help=_('Remote VTEP Tunnel IP'))
        parser.add_argument(
            '--segmentation-id', dest='segmentation_id',
            required=True,
            help=_('VNI to be used'))

    def args2body(self, args):
        body = {}
        attributes = ['mac_address', 'vtep_address', 'segmentation_id']
        gw_deviceV20.update_dict(args, body, attributes)
        return {'remote_mac_entry': body}


class RemoteMacEntryList(extension.ClientExtensionList, RemoteMacEntry):
    """List Gateway Device Remote Mac Entries."""

    shell_command = 'gateway-device-remote-mac-entry-list'
    list_columns = ['id', 'mac_address', 'vtep_address', 'segmentation_id']
    pagination_support = True
    sorting_support = True


class RemoteMacEntryShow(extension.ClientExtensionShow, RemoteMacEntry):
    """Show information of a given gateway-device-remote-mac-entry."""

    shell_command = 'gateway-device-remote-mac-entry-show'
    allow_names = False


class RemoteMacEntryDelete(extension.ClientExtensionDelete, RemoteMacEntry):
    """Delete a given gateway-device-remote-mac-entry."""

    shell_command = 'gateway-device-remote-mac-entry-delete'
    allow_names = False



# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sqlalchemy as sa

from neutron_lib.db import model_base


class BgpSpeakerRouterAssociation(model_base.BASEV2):

    """Tracks Bgp Speaker Router Association"""

    __tablename__ = 'bgp_speaker_router_associations'

    bgp_speaker_id = sa.Column(
        sa.String(36),
        sa.ForeignKey('bgp_speakers.id', ondelete="CASCADE"),
        nullable=False, primary_key=True)
    router_id = sa.Column(
        sa.String(36),
        sa.ForeignKey('routers.id', ondelete="CASCADE"),
        nullable=False, unique=True)



# Copyright 2015 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add data sync

Revision ID: d8a5c672761
Revises: 25aeae45d4ad
Create Date: 2015-03-16 06:04:40.695379

"""

# revision identifiers, used by Alembic.
revision = 'd8a5c672761'
down_revision = '25aeae45d4ad'

import datetime

from alembic import op
import sqlalchemy as sa


def upgrade():

    op.create_table('midonet_data_versions',
                    sa.Column('id', sa.Integer(), primary_key=True),
                    sa.Column('sync_started_at', sa.DateTime()),
                    sa.Column('sync_finished_at', sa.DateTime()),
                    sa.Column('sync_status', sa.String(length=50)),
                    sa.Column('sync_tasks_status', sa.String(length=50)),
                    sa.Column('stale', sa.Boolean(), nullable=False))

    op.create_table('midonet_data_state',
                    sa.Column('id', sa.Integer(), primary_key=True),
                    sa.Column('last_processed_task_id', sa.Integer()),
                    sa.Column('updated_at', sa.DateTime(),
                              default=datetime.datetime.utcnow,
                              nullable=False),
                    sa.Column('active_version', sa.Integer()),
                    sa.Column('readonly', sa.Boolean(), nullable=False),
                    sa.ForeignKeyConstraint(
                        ['last_processed_task_id'], ['midonet_tasks.id']),
                    sa.ForeignKeyConstraint(
                        ['active_version'],
                        ['midonet_data_versions.id']))

    op.execute(  # nosec
        "INSERT INTO midonet_data_state (id, last_processed_task_id, "
        "updated_at, active_version, readonly) VALUES (1, NULL, '%s', "
        "NULL, false)" % (datetime.datetime.utcnow()))



#
# Copyright 2016 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Rally Plugin to manage MidoNet resources"""

from __future__ import print_function

import netaddr
import random
import string
import utils

from rally.common import logging
from rally.task import atomic
from rally.task import scenario

AUTH_TOKEN = utils.RequestScenario._get_token()
LOG = logging.getLogger(__name__)


class MidonetRallyPluginMixin(object):

    def _execute_api(self, method, api, header, data=None):
        """Execute REST API call

        :param method: http method
        :param api: api for resource
        :param header: http header
        :param data: data fields for the resource
        :returns: HTTP status

        Depending on method provided, it executes appropriate REST call.
        If token expires, it regenerates auth token.
        """
        if method == "GET_ALL":
            val = self.request_get_all(api, header)
            if val != 401:
                return val
        elif method == "POST":
            val = self.request_post(api, header, data)
        elif method == "PUT":
            val = self.request_put(api, header, data)
        elif method == "GET_RESOURCE":
            val = self.request_get_resource(api, header)
        else:
            val = self.request_delete(api, header)
        if val == 401:
            global AUTH_TOKEN
            LOG.debug("Authentication token expired..!!")
            LOG.debug("Retrieving new Auth token and continuing operations")
            AUTH_TOKEN = utils.RequestScenario._get_token()
            header["X-Auth-Token"] = AUTH_TOKEN
            val = self._execute_api(method, api, header, data)
            LOG.debug("Executed value: ", val)
            return val

    @atomic.action_timer("create_bridge")
    def _create_bridge(self, method, api, header, data):
        """Benchmark create MidoNet bridge with REST API

        :param method: http method
        :param api: api for bridge
        :param header: http header
        :param data: data fields for the resource
        """
        self._execute_api(method, api, header, data)

    @atomic.action_timer("delete_bridge")
    def _delete_bridge(self, method, api, header, data):
        """Benchmark delete MidoNet bridge through REST API

        :param method: http method
        :param api: api for MidoNet bridge
        :param header: http header
        :param data: data fields for the resource
        """
        self._execute_api(method, api, header)

    @atomic.action_timer("create_router")
    def _create_router(self, method, api, header, data):
        """Benchmark create MidoNet router with REST API

        :param method: http method
        :param api: api for router
        :param header: http header
        :param data: data fields for the resource
        """
        self._execute_api(method, api, header, data)

    @atomic.action_timer("delete_router")
    def _delete_router(self, method, api, header, data):
        """Benchmark delete MidoNet router with REST API

        :param method: http method
        :param api: api for router
        :param header: http header
        :param data: data fields for the resource
        """
        self._execute_api(method, api, header, data)

    def _get_or_create_midonet_router(self, media_type):
        """Get the details of MidoNet routers

        It gets the list of all available MidoNet routers and parse its output
        to get the value of router id. In case if no router is available then
        new router is created.

        :param media_type: media type required for MidoNet router
        :returns : router id
        """
        router_api = "routers"
        col_media_type = media_type[:28] + 'collection.' + media_type[28:]
        header_get = {"Accept": col_media_type, "X-Auth-Token": "%s"
                      % AUTH_TOKEN}
        router_details = self._execute_api("GET_ALL", router_api, header_get)
        if not len(router_details) == 0:
            router_id = router_details[0]["id"]
        else:
            header_post = {"Content-Type": media_type, "X-Auth-Token": "%s"
                           % AUTH_TOKEN}
            # create a random name for router
            router_name = ''.join(random.choice(string.ascii_lowercase)
                                  for x in range(6))
            data = {"name": router_name, "tenantId": ""}
            # create router
            self._execute_api("POST", router_api, header_post, data)

            # get the router id
            router_details = self._execute_api("GET_ALL", router_api,
                                               header_get)
            router_id = router_details[0]["id"]

        return router_id

    @atomic.action_timer("create_router_port")
    def _create_router_port(self, method, api, header, data):
        """Benchmark create MidoNet port with REST API

        :param method: http method
        :param api: api for port
        :param header: http header
        :param data: data fields for the resource
        """
        self._execute_api(method, api, header, data)

    @atomic.action_timer("delete_router_port")
    def _delete_router_port(self, method, api, header, data):
        """Benchmark delete MidoNet port with REST API

        :param method: http method
        :param api: api for port
        :param header: http header
        :param data: data fields for the resource
        """
        self._execute_api(method, api, header, data)


@scenario.configure(name="MidonetRallyPlugin.create_midonet_bridge")
class CreateMidonetBridge(MidonetRallyPluginMixin, utils.RequestScenario):

    def run(self, api, media_type, no_of_bridges):
        """Create MidoNet bridge with REST API

        :param api: api for MidoNet bridge
        :param media_type: media type required for MidoNet bridge
        :param no_of_bridges: number of bridges to be created in one iteration
        """
        # header for bridge API
        header = {"Content-Type": media_type, "X-Auth-Token": "%s" %
                                                              AUTH_TOKEN}
        for _ in range(no_of_bridges):
            # generate payload for creating bridge
            # creating bridge requires bridge name
            # bridge name is generated randomly
            bridge_name = ''.join(random.choice(string.ascii_lowercase) for x
                                  in range(10))
            data = {"name": bridge_name, "tenantId": ""}
            # create bridge
            self._create_bridge("POST", api, header, data)


@scenario.configure(name="MidonetRallyPlugin.delete_midonet_bridge")
class DeleteMidonetBridge(MidonetRallyPluginMixin, utils.RequestScenario):

    def run(self, api, media_type):
        """Deletes all available bridges

        :param api: api for MidoNet bridge
        :param media_type: media type required for MidoNet bridge
        """
        # to delete all the bridges, retrieve IDs of bridge
        # header for router GET command
        header = {"X-Auth-Token": "%s" % AUTH_TOKEN}
        bridge_details = self._execute_api("GET_ALL", api, header)

        # parse the result of bridge GET command
        # to get only IDs of bridge
        bridge_ids = [bridge["id"] for bridge in bridge_details]
        LOG.debug("Number of bridges to be deleted are: %s" % len(bridge_ids))

        # update header for delete API
        header["Content-Type"] = media_type
        # delete midonet routers
        [self._delete_bridge("DELETE", api + "/" + bridge_id, header)
         for bridge_id in bridge_ids]


@scenario.configure(name="MidonetRallyPlugin.create_midonet_router")
class CreateMidonetRouter(MidonetRallyPluginMixin, utils.RequestScenario):

    def run(self, api, media_type, no_of_routers):
        """Create MidoNet router with REST API

        :param api: api for creating MidoNet router
        :param media_type: media type for creating MidoNet router
        :param no_of_routers: number of routers to be created in one iteration
        """

        # header for router API
        header = {"Content-Type": media_type, "X-Auth-Token": "%s" %
                                                              AUTH_TOKEN}
        for _ in range(no_of_routers):
            # generate payload for creating router
            # creating router requires router name
            # router name is generated randomly
            router_name = ''.join(random.choice(string.ascii_lowercase)
                                  for x in range(10))
            data = {"name": router_name, "tenantId": ""}
            # create router
            self._create_router("POST", api, header, data)


@scenario.configure(name="MidonetRallyPlugin.delete_midonet_router")
class DeleteMidonetRouter(MidonetRallyPluginMixin, utils.RequestScenario):

    def run(self, api, media_type):
        """Deletes all available routers

        :param api: api for MidoNet router
        :param media_type: media type required for MidoNet router
        """

        # to delete all the routers, retrieve IDs of router
        col_media_type = media_type[:28] + 'collection.' + media_type[28:]

        # header for router GET command
        header_get = {"Accept": col_media_type, "X-Auth-Token": "%s"
                      % AUTH_TOKEN}
        router_details = self._execute_api("GET_ALL", api, header_get)
        # parse the result of router GET command
        # to get only IDs of router
        router_ids = [router["id"] for router in router_details]
        LOG.debug("Number of routers to be deleted are: %s" % len(router_ids))

        # update header for delete API
        header_del = {"Content-Type": media_type, "X-Auth-Token": "%s" %
                      AUTH_TOKEN}
        # delete midonet routers
        [self._delete_router("DELETE", api + "/" + router_id, header_del)
         for router_id in router_ids]


@scenario.configure(name="MidonetRallyPlugin.create_midonet_router_port")
class CreateMidonetRouterPort(MidonetRallyPluginMixin, utils.RequestScenario):

    def run(self, api, media_type, data, no_of_ports):
        """Create port on MidoNet router

        :param api: api for MidoNet port
        :param media_type: media type required for MidoNet port
        :param data: payload data for MidoNet router port
        :param no_of_ports: number of ports to be created at one time
        """
        # router id is retrieved with router get command
        # the first router id available is used for the operation
        router_id = self._get_or_create_midonet_router(media_type['router'])
        post_api = "routers/" + router_id + "/" + api
        # set header with content-type and authentication token
        header = {"Content-Type": media_type['port'], "X-Auth-Token": "%s"
                  % AUTH_TOKEN}
        cidr = data["networkAddress"] + '/' + data["networkLength"]
        ip_list = netaddr.IPNetwork(cidr)

        for _ in range(no_of_ports):
            # port address is generated randomly in the cidr
            port_address = str(random.choice(ip_list))
            LOG.debug("port_address is: %s" % port_address)
            data["portAddress"] = port_address
            # create port
            self._create_router_port("POST", post_api, header, data)


@scenario.configure(name="MidonetRallyPlugin.delete_midonet_port")
class DeleteMidonetPort(MidonetRallyPluginMixin, utils.RequestScenario):

    def run(self, api, media_type):
        """Deletes all available ports

        :param api: api for MidoNet port
        :param media_type: media type required for MidoNet port
        """

        col_media_type = media_type['port'][:28] + 'collection.' \
            + media_type['port'][28:]
        # header for get command
        header_get = {"Accept": col_media_type, "X-Auth-Token": "%s"
                                                                % AUTH_TOKEN}
        port_details = self._execute_api("GET_ALL", api, header_get)

        # get the all available ports
        ports = [port["id"] for port in port_details]
        LOG.debug("Number of ports to be deleted are: %s" % len(ports))

        # update header for delete API
        header_del = {"Content-Type": media_type['port'], "X-Auth-Token": "%s"
                      % AUTH_TOKEN}
        # delete midonet ports
        [self._delete_router_port("DELETE", api + "/" + port_id, header_del)
         for port_id in ports]



# Copyright 2015 OpenStack Foundation
# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.api import validators
from neutron_lib import exceptions

from networking_l2gw.services.l2gateway.common import constants as l2gw_const
from networking_l2gw.services.l2gateway.common import l2gw_validators

from midonet.neutron._i18n import _
from midonet.neutron.common import constants
from midonet.neutron.extensions import gateway_device


def validate_gwdevice_list(data, valid_values=None):
    """Validate the list of devices."""

    if not data:
        # Devices must be provided
        msg = _("Cannot create a gateway with an empty device list")
        return msg

    if len(data) > 1:
        # The number of devices must be exactly one
        msg = _("Exactly one device must be specified to create a gateway")
        return msg

    try:
        for device in data:
            err_msg = validators.validate_dict(device, None)
            if err_msg:
                return err_msg

            device_id = device.get('device_id')
            if not device_id:
                msg = _("Cannot create a gateway with an empty device_id")
                return msg

            # Don't accept any interface.  However, when supporting HW VTEPs
            # this must be supported.
            # TODO(RYU): Allow setting segmentation ID in some way
            if device.get('interfaces'):
                msg = _("Interfaces are not allowed in MidoNet L2GW")
                return msg
            device['interfaces'] = []
    except TypeError:
        return (_("%s: provided data are not iterable") %
                validate_gwdevice_list.__name__)


def validate_network_mapping_list_without_seg_id_validation(network_mapping,
                                                            check_vlan):
    """Validate network mapping list in connection."""
    if network_mapping.get('segmentation_id'):
        if check_vlan:
            raise exceptions.InvalidInput(
                error_message=_("default segmentation_id should not be"
                                " provided when segmentation_id is assigned"
                                " during l2gateway creation"))
        # This method doen't check segmentation id range.

    if not network_mapping.get('segmentation_id'):
        if check_vlan is False:
            raise exceptions.InvalidInput(
                error_message=_("Segmentation id must be specified in create "
                                "l2gateway connections"))
    network_id = network_mapping.get(l2gw_const.NETWORK_ID)
    if not network_id:
        raise exceptions.InvalidInput(
            error_message=_("A valid network identifier must be specified "
                            "when connecting a network to a network "
                            "gateway. Unable to complete operation"))
    connection_attrs = set(network_mapping.keys())
    if not connection_attrs.issubset(l2gw_validators.
                                     ALLOWED_CONNECTION_ATTRIBUTES):
        raise exceptions.InvalidInput(
            error_message=(_("Invalid keys found among the ones provided "
                             "in request : %(connection_attrs)s."),
                           connection_attrs))
    return network_id


def is_valid_segmentaion_id(gw_type, seg_id):
    if (gw_type == gateway_device.ROUTER_DEVICE_TYPE):
        is_valid_vxlan_id(seg_id)
    elif gw_type == gateway_device.NETWORK_VLAN_TYPE:
        l2gw_validators.is_valid_vlan_id(seg_id)


def is_valid_vxlan_id(seg_id):
    try:
        int_seg_id = int(seg_id)
    except ValueError:
        msg = _("Segmentation id must be a valid integer")
        raise exceptions.InvalidInput(error_message=msg)
    if int_seg_id < 0 or int_seg_id >= constants.MAX_VXLAN_VNI:
        msg = _("Segmentation id is out of range")
        raise exceptions.InvalidInput(error_message=msg)



# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from neutronclient import shell

from midonet.neutron.tests.unit.neutronclient_ext import test_cli20
from midonet.neutronclient.l2gateway_extension import _l2_gateway


class CLITestV20L2gatewayJSON(test_cli20.CLIExtTestV20Base):

    def setUp(self):
        l2_gateway = ("l2_gateway", _l2_gateway)
        self._mock_load_extensions(l2_gateway)
        super(CLITestV20L2gatewayJSON,
              self).setUp(plurals={'l2_gateways': 'l2_gateway'})
        self.register_non_admin_status_resource('l2_gateway')

    def test_midonet_l2gateway_cmd_loaded(self):
        neutron_shell = shell.NeutronShell('2.0')
        mido_l2gw_cmd = {'midonet-l2-gateway-create':
                         _l2_gateway.L2GatewayCreate,
                         }
        for cmd_name, cmd_class in mido_l2gw_cmd.items():
            found = neutron_shell.command_manager.find_command([cmd_name])
            self.assertEqual(cmd_class, found[0])

    def _create_l2gateway(self, name, args,
                          position_names, position_values):
        resource = 'l2_gateway'
        cmd = _l2_gateway.L2GatewayCreate(test_cli20.MyApp(sys.stdout), None)
        self._test_create_resource(resource, cmd, name, 'myid',
                                   args, position_names, position_values)

    def test_create_l2gateway(self):
        name = 'l2gateway1'
        args = [name, '--device',
                'device_id=my_device_id,segmentation_id=my_segmentation_id']
        position_names = ['name', 'devices']
        position_values = [name, [{"device_id": "my_device_id",
                                  "segmentation_id": "my_segmentation_id"}]]
        self._create_l2gateway(name, args,
                               position_names, position_values)

    def test_create_l2gateway_with_multiple_devices(self):
        name = 'l2gateway1'
        args = [name,
                '--device',
                'device_id=my_device_id1,segmentation_id=my_segmentation_id1',
                '--device',
                'device_id=my_device_id2,segmentation_id=my_segmentation_id2']
        position_names = ['name', 'devices']
        position_values = [name,
                           [{"device_id": "my_device_id1",
                             "segmentation_id": "my_segmentation_id1"},
                            {"device_id": "my_device_id2",
                             "segmentation_id": "my_segmentation_id2"}]]
        self._create_l2gateway(name, args,
                               position_names, position_values)

    def test_create_l2gateway_without_segmentation_id(self):
        name = 'l2gateway1'
        args = [name, '--device', 'device_id=my_device_id']
        position_names = ['name', 'devices']
        position_values = [name, [{"device_id": "my_device_id"}]]
        self._create_l2gateway(name, args,
                               position_names, position_values)



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import neutron_lib.exceptions as exc

from midonet.neutron._i18n import _


class ClusterConnectionError(exc.ServiceUnavailable):
    message = _("Error connecting to cluster")


class MidonetApiException(exc.NeutronException):
    message = _("MidoNet API error: %(msg)s")


class InvalidMidonetDataState(exc.NeutronException):
    """Exception to signify a state in the midonet tables that is invalid.

    i.e. missing some table that should always be present.
    """
    pass



# Copyright (C) 2016,2017 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import portbindings
from neutron_lib import constants
from neutron_lib.db import constants as db_consts
from neutron_lib.services.qos import base
from neutron_lib.services.qos import constants as qos_consts
from oslo_config import cfg
from oslo_log import helpers as log_helpers
from oslo_log import log as logging

from midonet.neutron.client import base as c_base
from midonet.neutron.common import config  # noqa
from midonet.neutron.common import constants as m_const


LOG = logging.getLogger(__name__)


DRIVER = None


SUPPORTED_RULES = {
    qos_consts.RULE_TYPE_BANDWIDTH_LIMIT: {
        qos_consts.MAX_KBPS: {
            'type:range': [0, db_consts.DB_INTEGER_MAX_VALUE]},
        qos_consts.MAX_BURST: {
            'type:range': [0, db_consts.DB_INTEGER_MAX_VALUE]},
        qos_consts.DIRECTION: {
            'type:values': [constants.EGRESS_DIRECTION]}
    },
    qos_consts.RULE_TYPE_DSCP_MARKING: {
        qos_consts.DSCP_MARK: {'type:values': constants.VALID_DSCP_MARKS}
    }
}


# TODO(yamamoto): Override precommit methods for task-based api
class MidoNetQosDriver(base.DriverBase):

    def __init__(self, *args, **kwargs):
        super(MidoNetQosDriver, self).__init__(*args, **kwargs)
        self._client = c_base.load_client(cfg.CONF.MIDONET)

    @staticmethod
    @log_helpers.log_method_call
    def create():
        return MidoNetQosDriver(
            name='midonet',
            vif_types=[
                m_const.VIF_TYPE_MIDONET,
            ],
            vnic_types=[
                portbindings.VNIC_NORMAL,
            ],
            supported_rules=SUPPORTED_RULES,
            requires_rpc_notifications=False)

    @log_helpers.log_method_call
    def create_policy(self, context, policy):
        self._client.create_qos_policy(context, policy)

    @log_helpers.log_method_call
    def update_policy(self, context, policy):
        self._client.update_qos_policy(context, policy)

    @log_helpers.log_method_call
    def delete_policy(self, context, policy):
        self._client.delete_qos_policy(context, policy)


def register():
    global DRIVER
    if not DRIVER:
        DRIVER = MidoNetQosDriver.create()
    LOG.debug('MidoNet QoS driver registered')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from os_vif import objects
from oslo_concurrency import processutils

from midonet.os_vif import linux_net
from midonet.os_vif import privsep
from midonet.os_vif import vif_midonet


class PluginTest(testtools.TestCase):

    def setUp(self):
        super(PluginTest, self).setUp()
        privsep.mm_ctl.set_client_mode(False)

    def __init__(self, *args, **kwargs):
        super(PluginTest, self).__init__(*args, **kwargs)

        objects.register_all()

        self.subnet_bridge_4 = objects.subnet.Subnet(
            cidr='101.168.1.0/24',
            dns=['8.8.8.8'],
            gateway='101.168.1.1',
            dhcp_server='191.168.1.1')

        self.subnet_bridge_6 = objects.subnet.Subnet(
            cidr='101:1db9::/64',
            gateway='101:1db9::1')

        self.subnets = objects.subnet.SubnetList(
            objects=[self.subnet_bridge_4,
                     self.subnet_bridge_6])

        self.network = objects.network.Network(
            id='437c6db5-4e6f-4b43-b64b-ed6a11ee5ba7',
            subnets=self.subnets)

        self.vif = objects.vif.VIFGeneric(
            id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
            address='ca:fe:de:ad:be:ef',
            network=self.network,
            vif_name='tap-xxx-yyy-zzz')

        self.instance = objects.instance_info.InstanceInfo(
            name='demo',
            uuid='f0000000-0000-0000-0000-000000000001')

    @mock.patch.object(linux_net, 'create_tap_dev')
    @mock.patch.object(processutils, 'execute')
    def test_plug(self, execute, create_tap_dev):
        plugin = vif_midonet.MidoNetPlugin.load("midonet")
        plugin.plug(self.vif, self.instance)
        create_tap_dev.assert_called_once_with('tap-xxx-yyy-zzz')
        execute.assert_called_once_with(
            'mm-ctl', '--bind-port',
            'b679325f-ca89-4ee0-a8be-6db1409b69ea', 'tap-xxx-yyy-zzz')

    @mock.patch.object(linux_net, 'delete_net_dev')
    @mock.patch.object(processutils, 'execute')
    def test_unplug(self, execute, delete_net_dev):
        plugin = vif_midonet.MidoNetPlugin.load("midonet")
        plugin.unplug(self.vif, self.instance)
        delete_net_dev.assert_called_once_with('tap-xxx-yyy-zzz')
        execute.assert_called_once_with(
            'mm-ctl', '--unbind-port', 'b679325f-ca89-4ee0-a8be-6db1409b69ea')



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from neutron_lib import constants
from neutron_lib.plugins import constants as plugin_const
import webob.exc

from neutron.db import servicetype_db as sdb
from neutron import extensions as nextensions
from neutron.services import provider_configuration as provconf
from neutron.tests.unit.extensions import test_l3 as test_l3_plugin
from neutron_vpnaas import extensions
from neutron_vpnaas.tests.unit.db.vpn import test_vpn_db

from midonet.neutron.tests.unit import test_midonet_plugin_ml2 as test_mn_ml2


MN_DRIVER_KLASS = ('midonet.neutron.services.vpn.service_drivers.'
                   'midonet_ipsec.MidonetIPsecVPNDriver')

extensions_path = ':'.join(extensions.__path__ + nextensions.__path__)
DB_VPN_PLUGIN_KLASS = "neutron_vpnaas.services.vpn.plugin.VPNDriverPlugin"
FLAVOR_PLUGIN_KLASS = "neutron.services.flavors.flavors_plugin.FlavorsPlugin"


class VPNTestExtensionManager(test_l3_plugin.L3TestExtensionManager):

    def get_resources(self):
        res = super(VPNTestExtensionManager, self).get_resources()
        return res + extensions.vpnaas.Vpnaas.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class VPNTestCaseMixin(test_vpn_db.VPNTestMixin,
                       test_l3_plugin.L3NatTestCaseMixin):
    def setUp(self):
        service_plugins = {
            'vpn_plugin_name': DB_VPN_PLUGIN_KLASS,
            'flavors_plugin': FLAVOR_PLUGIN_KLASS}
        vpnaas_provider = (plugin_const.VPN + ':vpnaas:' + MN_DRIVER_KLASS
                           + ':default')
        mock.patch.object(provconf.NeutronModule, 'service_providers',
                          return_value=[vpnaas_provider]).start()
        manager = sdb.ServiceTypeManager.get_instance()
        manager.add_provider_configuration(
            plugin_const.VPN, provconf.ProviderConfiguration())

        super(VPNTestCaseMixin, self).setUp(service_plugins=service_plugins,
                                            ext_mgr=VPNTestExtensionManager())

    def test_create_vpn_service(self):
        with self.vpnservice() as vpnservice:
            req = self.new_show_request(
                'vpnservices', vpnservice['vpnservice']['id'])
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertEqual(constants.ACTIVE, res['vpnservice']['status'])

    def test_create_vpn_service_error_delete_neutron_resource(self):
        self.client_mock.create_vpn_service.side_effect = Exception(
            "Fake Error")
        with self.subnet(cidr='10.2.0.0/24') as subnet, \
                self.router() as router:
            try:
                with self.vpnservice(subnet=subnet, router=router,
                                     do_delete=False):
                    # Shouldn't be reached
                    self.assertTrue(False)
            except webob.exc.HTTPClientError:
                pass
            req = self.new_list_request('vpnservices')
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertFalse(res['vpnservices'])

    def test_update_vpn_service(self):
        with self.vpnservice() as vpnservice:
            data = {'vpnservice': {'name': 'vpnservice2'}}
            vpnservice_id = vpnservice['vpnservice']['id']
            req = self.new_update_request('vpnservices', data, vpnservice_id)
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertEqual('vpnservice2', res['vpnservice']['name'])
            self.assertEqual(
                'vpnservice2',
                self.client_mock.update_vpn_service.call_args[0][2]['name'])

    def test_update_vpn_service_error_change_neutron_resource_status(self):
        self.client_mock.update_vpn_service.side_effect = Exception(
            "Fake Error")
        with self.vpnservice() as vpnservice:
            data = {'vpnservice': {'name': 'vpnservice2'}}
            vpnservice_id = vpnservice['vpnservice']['id']
            req = self.new_update_request('vpnservices', data, vpnservice_id)
            res = req.get_response(self.ext_api)
            self.assertEqual(500, res.status_int)

            req = self.new_show_request('vpnservices', vpnservice_id)
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertEqual(constants.ERROR, res['vpnservice']['status'])

    def test_delete_vpnservice(self):
        """Test case to delete a vpnservice."""
        with self.vpnservice(name='vpnserver',
                             do_delete=False) as vpnservice:
            req = self.new_delete_request('vpnservices',
                                          vpnservice['vpnservice']['id'])
            res = req.get_response(self.ext_api)
            self.assertEqual(204, res.status_int)

    def test_delete_vpn_service_error_delete_neutron_resouce(self):
        self.client_mock.delete_vpn_service_side_effect = Exception(
            "Fake Error")
        self.test_delete_vpnservice()
        # check the resouce deleted in Neutron DB
        req = self.new_list_request('vpnservices')
        res = self.deserialize(self.fmt, req.get_response(self.ext_api))
        self.assertFalse(res['vpnservices'])

    def test_create_ipsec_site_connection(self):
        with self.ipsec_site_connection() as ipsec_site_connection:
            req = self.new_show_request(
                'ipsec-site-connections',
                ipsec_site_connection['ipsec_site_connection']['id'])
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertEqual(
                constants.ACTIVE,
                res['ipsec_site_connection']['status'])

    def test_create_two_ipsec_site_connections_one_vpnservice(self):
        with self.vpnservice() as vpnservice, \
                self.ipsec_site_connection(vpnservice=vpnservice), \
                self.ipsec_site_connection(vpnservice=vpnservice,
                                           peer_address='192.168.1.11',
                                           peer_id='192.168.1.11',
                                           peer_cidrs=['10.0.11.0/24']):
            # Check there are two ipsec site connections
            req = self.new_list_request('ipsec-site-connections')
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertTrue(len(res['ipsec_site_connections']) == 2)
            self.assertNotEqual(res['ipsec_site_connections'][0]['id'],
                                res['ipsec_site_connections'][1]['id'])

            for ipsec_site_connection in res['ipsec_site_connections']:
                # Check that the associated vpnservice is the correct one
                req = self.new_show_request(
                    'vpnservices', ipsec_site_connection['vpnservice_id'])
                res = self.deserialize(self.fmt,
                                       req.get_response(self.ext_api))
                self.assertEqual(vpnservice['vpnservice']['id'],
                                 res['vpnservice']['id'])

                self.assertEqual(constants.ACTIVE,
                                 ipsec_site_connection['status'])

    def test_create_ipsec_site_connection_error_delete_neutron_resouce(self):
        self.client_mock.create_ipsec_site_conn.side_effect = Exception(
            "Fake Error")
        with self.vpnservice() as vpnservice, \
                self.ikepolicy() as ikepolicy, \
                self.ipsecpolicy() as ipsecpolicy:
            self._create_ipsec_site_connection(
                self.fmt, 'site_conn2',
                peer_cidrs='192.168.101.0/24',
                vpnservice_id=vpnservice['vpnservice']['id'],
                ikepolicy_id=ikepolicy['ikepolicy']['id'],
                ipsecpolicy_id=ipsecpolicy['ipsecpolicy']['id'],
                expected_res_status=500)
            req = self.new_list_request('ipsec-site-connections')
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertFalse(res['ipsec_site_connections'])

    def test_update_ipsec_site_connection(self):
        with self.ipsec_site_connection() as ipsec_site_connection:
            data = {'ipsec_site_connection': {'mtu': '1300'}}
            ipsec_site_conn_id = (
                ipsec_site_connection['ipsec_site_connection']['id'])
            req = self.new_update_request(
                'ipsec-site-connections', data, ipsec_site_conn_id)
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertEqual(1300, res['ipsec_site_connection']['mtu'])
            self.assertEqual(
                1300,
                self.client_mock.update_ipsec_site_conn.call_args[0][2]['mtu'])

    def test_update_ipsec_site_connection_error(self):
        self.client_mock.update_ipsec_site_conn.side_effect = Exception(
            "Fake Error")
        with self.ipsec_site_connection() as ipsec_site_connection:
            data = {'ipsec_site_connection': {'mtu': '1300'}}
            ipsec_site_conn_id = (
                ipsec_site_connection['ipsec_site_connection']['id'])
            req = self.new_update_request('ipsec-site-connections', data,
                                          ipsec_site_conn_id)
            res = req.get_response(self.ext_api)
            self.assertEqual(500, res.status_int)

            req = self.new_show_request(
                'ipsec-site-connections',
                ipsec_site_conn_id)
            res = self.deserialize(self.fmt, req.get_response(self.ext_api))
            self.assertEqual(
                constants.ERROR,
                res['ipsec_site_connection']['status'])

    def test_delete_ipsec_site_connection(self):
        with self.ipsec_site_connection(
                name="site_conn2", do_delete=False) as ipsec_site_connection:
            ipsec_site_conn_id = \
                ipsec_site_connection['ipsec_site_connection']['id']
            req = self.new_delete_request(
                'ipsec-site-connections', ipsec_site_conn_id)
            res = req.get_response(self.ext_api)
            self.assertEqual(204, res.status_int)

    def test_delete_ipsec_site_connection_error(self):
        self.client_mock.delete_ipsec_site_conn.side_effect = Exception(
            "Fake Error")
        self.test_delete_ipsec_site_connection()
        req = self.new_list_request('ipsec-site-connections')
        res = self.deserialize(self.fmt, req.get_response(self.ext_api))
        self.assertFalse(res['ipsec_site_connections'])


class VPNTestCaseML2(VPNTestCaseMixin,
                     test_mn_ml2.MidonetPluginML2TestCase):
    pass



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import helpers as log_helpers
from oslo_log import log as logging
from oslo_utils import excutils

from neutron.api import extensions as neutron_extensions

from midonet.neutron.client import base as c_base
from midonet.neutron.common import config  # noqa
from midonet.neutron.common import constants as midonet_const
from midonet.neutron.db import gateway_device as gateway_device_db
from midonet.neutron import extensions
from midonet.neutron.extensions import gateway_device

LOG = logging.getLogger(__name__)


class MidonetGwDeviceServicePlugin(gateway_device_db.GwDeviceDbMixin):

    """Implements GatewayDevice service plugin for Midonet."""

    supported_extension_aliases = ["gateway-device"]

    def __init__(self):
        super(MidonetGwDeviceServicePlugin, self).__init__()

        # Instantiate MidoNet API client
        self.client = c_base.load_client(cfg.CONF.MIDONET)

        neutron_extensions.append_api_extensions_path(extensions.__path__)

    @classmethod
    def get_plugin_type(cls):
        return midonet_const.GATEWAY_DEVICE

    def get_plugin_description(self):
        """Returns string description of the plugin."""
        return "Midonet Gateway Device Service Plugin"

    @log_helpers.log_method_call
    def create_gateway_device(self, context, gateway_device):

        with context.session.begin(subtransactions=True):
            gw = super(MidonetGwDeviceServicePlugin,
                       self).create_gateway_device(context, gateway_device)
            self.client.create_gateway_device_precommit(context, gw)

        try:
            self.client.create_gateway_device_postcommit(gw)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error("Failed to create a gateway "
                          "device %(gw_id)s in Midonet: %(err)s",
                          {"gw_id": gw["id"], "err": ex})
                try:
                    self.delete_gateway_device(context, gw['id'])
                except Exception:
                    LOG.exception("Failed to delete a gateway device %s",
                                  gw["id"])
        return gw

    @log_helpers.log_method_call
    def update_gateway_device(self, context, id, gateway_device):
        backup = self.get_gateway_device(context, id)
        del backup['id']
        del backup['remote_mac_entries']
        backup_body = {'gateway_device': backup}
        with context.session.begin(subtransactions=True):
            gw = super(MidonetGwDeviceServicePlugin,
                       self).update_gateway_device(context, id, gateway_device)
            self.client.update_gateway_device_precommit(context, id, gw)

        try:
            self.client.update_gateway_device_postcommit(id, gw)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error("Failed to update a gateway "
                          "device %(gw_id)s in Midonet:%(err)s",
                          {"gw_id": gw["id"], "err": ex})
                try:
                    super(
                        MidonetGwDeviceServicePlugin,
                        self).update_gateway_device(
                            context, gw['id'], backup_body)
                except Exception:
                    LOG.exception("Failed to update a gateway "
                                  "device for rollback %s", gw["id"])
        return gw

    @log_helpers.log_method_call
    def delete_gateway_device(self, context, id):
        with context.session.begin(subtransactions=True):
            super(MidonetGwDeviceServicePlugin,
                  self).delete_gateway_device(context, id)
            self.client.delete_gateway_device_precommit(context, id)

        self.client.delete_gateway_device_postcommit(id)

    @log_helpers.log_method_call
    def create_gateway_device_remote_mac_entry(self, context,
                                               remote_mac_entry,
                                               gateway_device_id):
        gw_device = self._get_gateway_device(context, gateway_device_id)

        if gw_device.type == gateway_device.NETWORK_VLAN_TYPE:
            raise gateway_device.OperationRemoteMacEntryNotSupported(
                type=gateway_device.NETWORK_VLAN_TYPE)

        with context.session.begin(subtransactions=True):
            rme = super(MidonetGwDeviceServicePlugin,
                        self).create_gateway_device_remote_mac_entry(
                context, gateway_device_id, remote_mac_entry)
            self.client.create_gateway_device_remote_mac_entry_precommit(
                context, rme)

        try:
            self.client.create_gateway_device_remote_mac_entry_postcommit(rme)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error("Failed to create a remote mac entry "
                          "%(rme_id)s for %(gw_id)s in Midonet:%(err)s",
                          {"rme_id": rme["id"], "gw_id": gateway_device_id,
                           "err": ex})
                try:
                    super(MidonetGwDeviceServicePlugin,
                          self).delete_gateway_device_remote_mac_entry(
                        context, rme["id"], gateway_device_id)
                except Exception:
                    LOG.exception("Failed to delete a remote mac entry %s",
                                  rme["id"])

        return rme

    @log_helpers.log_method_call
    def delete_gateway_device_remote_mac_entry(self, context, id,
                                               gateway_device_id):
        self._get_gateway_device(context, gateway_device_id)
        with context.session.begin(subtransactions=True):
            super(MidonetGwDeviceServicePlugin,
                  self).delete_gateway_device_remote_mac_entry(
                context, id, gateway_device_id)
            self.client.delete_gateway_device_remote_mac_entry_precommit(
                context, id)

        self.client.delete_gateway_device_remote_mac_entry_postcommit(id)



# Copyright 2015 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Revert dynamic routing

Revision ID: 4f3b347ea1c2
Revises: 29cbb88b092
Create Date: 2015-12-04 15:01:34.026502

"""

# revision identifiers, used by Alembic.
revision = '4f3b347ea1c2'
down_revision = '29cbb88b092'

from alembic import op


def upgrade():
    op.drop_table('midonet_advertise_route')
    op.drop_table('midonet_routing_peers')
    op.drop_table('midonet_routing_instances')



# Copyright (C) 2016 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

from neutron_lib import exceptions as nexception
from neutron_lib.plugins import directory
from oslo_config import cfg
from oslo_log import helpers as log_helpers
from oslo_log import log as logging
from oslo_utils import excutils

from neutron.api import extensions as neutron_extensions
from neutron.db import api as db_api
from neutron_dynamic_routing import extensions as bgp_extensions
from neutron_dynamic_routing.extensions import bgp

from midonet.neutron.client import base as c_base
from midonet.neutron.common import config  # noqa
from midonet.neutron.common import constants as m_const
from midonet.neutron.db import bgp_db_midonet
from midonet.neutron.db import bgp_speaker_router_insertion_db as bsridb
from midonet.neutron import extensions
from midonet.neutron.extensions import bgp_speaker_router_insertion as bsri


LOG = logging.getLogger(__name__)


class MidonetBgpPlugin(bgp_db_midonet.MidonetBgpDbMixin,
                       bsridb.BgpSpeakerRouterInsertionDbMixin):

    """Implements MidoNet BGP dynamic routing service plugin.

    This class manages the workflow of Midonet BGP request/response.
    """

    supported_extension_aliases = [bgp.BGP_EXT_ALIAS,
                                   bsri.BGP_ROUTER_EXT_ALIAS]

    def __init__(self):
        neutron_extensions.append_api_extensions_path(extensions.__path__)
        neutron_extensions.append_api_extensions_path(bgp_extensions.__path__)

        # Instantiate MidoNet API client.
        self.client = c_base.load_client(cfg.CONF.MIDONET)

        super(MidonetBgpPlugin, self).__init__()

    @classmethod
    def get_plugin_type(cls):
        return bgp.BGP_EXT_ALIAS

    def get_plugin_description(self):
        """returns string description of the plugin."""
        return ("MidoNet BGP dynamic routing service")

    @log_helpers.log_method_call
    def create_bgp_speaker(self, context, bgp_speaker):
        with db_api.context_manager.writer.using(context):
            bgp_sp = super(MidonetBgpPlugin,
                           self).create_bgp_speaker(context, bgp_speaker)
            router_id = bgp_speaker['bgp_speaker'][m_const.LOGICAL_ROUTER]
            if router_id:
                # If the router is already associated with bgp-speaker,
                # RouterInUse will be raised.
                self.set_router_for_bgp_speaker(
                    context, bgp_sp['id'], router_id)
                bgp_sp[m_const.LOGICAL_ROUTER] = router_id

        return bgp_sp

    @log_helpers.log_method_call
    def delete_bgp_speaker(self, context, bgp_speaker_id):
        with db_api.context_manager.writer.using(context):
            bgp_sp = super(MidonetBgpPlugin, self).get_bgp_speaker(
                context, bgp_speaker_id)
            super(MidonetBgpPlugin, self).delete_bgp_speaker(
                context, bgp_speaker_id)
            # Plugin should call 'UPDATE' to clean up bgp-peers because
            # backend cannot handle excepting ID when 'DELETE' is called.
            self.client.update_bgp_speaker_precommit(
                context, bgp_speaker_id, bgp_sp)

        self.client.update_bgp_speaker_postcommit(
            bgp_speaker_id, bgp_sp)

    @log_helpers.log_method_call
    def get_bgp_speakers(self, context, filters=None, fields=None,
                         sorts=None, limit=None, marker=None,
                         page_reverse=False):
        bgp_sp_list = super(MidonetBgpPlugin, self).get_bgp_speakers(
            context, filters, fields)
        for bgp_sp in bgp_sp_list:
            rt_id = self.get_router_associated_with_bgp_speaker(
                context, bgp_sp['id'])
            bgp_sp[m_const.LOGICAL_ROUTER] = rt_id
        return bgp_sp_list

    @log_helpers.log_method_call
    def get_bgp_speaker(self, context, bgp_speaker_id, fields=None):
        bgp_sp = super(MidonetBgpPlugin, self).get_bgp_speaker(
            context, bgp_speaker_id, fields)
        rt_id = self.get_router_associated_with_bgp_speaker(
            context, bgp_speaker_id)
        bgp_sp[m_const.LOGICAL_ROUTER] = rt_id
        return bgp_sp

    @log_helpers.log_method_call
    def add_bgp_peer(self, context, bgp_speaker_id, bgp_peer_info):
        # TODO(kengo): This is temporary workaround until upstream raise
        # an error when dictionary without 'bgp_peer_id' key is specified.
        if not self._get_id_for(bgp_peer_info, 'bgp_peer_id'):
            raise nexception.BadRequest(
                resource=bgp.BGP_SPEAKER_RESOURCE_NAME,
                msg="bgp_peer_id must be specified")
        with db_api.context_manager.writer.using(context):
            # In MidoNet, bgp-peer can be related to only one bgp-speaker.
            if self._get_bgp_speakers_by_bgp_peer_binding(
                    context, bgp_peer_info['bgp_peer_id']):
                raise bsri.MidonetBgpPeerInUse(
                    id=bgp_peer_info['bgp_peer_id'])
            if not self.get_router_associated_with_bgp_speaker(
                    context, bgp_speaker_id):
                # External network must be associated with the bgp speaker.
                raise bsri.ExternalNetworkUnbound()
            info = super(MidonetBgpPlugin, self).add_bgp_peer(
                context, bgp_speaker_id, bgp_peer_info)
            # get peer info for MidoNet
            bgp_peer = super(MidonetBgpPlugin, self).get_bgp_peer(
                context, info['bgp_peer_id'])
            # merge bgp_speaker information for MidoNet
            bgp_peer['bgp_speaker'] = self.get_bgp_speaker(
                context, bgp_speaker_id)
            self.client.create_bgp_peer_precommit(context, bgp_peer)

        try:
            self.client.create_bgp_peer_postcommit(bgp_peer)
        except Exception as ex:
            LOG.error("Failed to create MidoNet resources to add bgp "
                      "peer. bgp_peer=%(bgp_peer)s, "
                      "bgp_speaker_id=%(bgp_speaker_id)s, error=%(err)r",
                      {"bgp_peer": bgp_peer, "bgp_speaker_id": bgp_speaker_id,
                       "err": ex})
            with excutils.save_and_reraise_exception():
                super(MidonetBgpPlugin, self).remove_bgp_peer(
                    context, bgp_speaker_id, bgp_peer_info)

        return info

    @log_helpers.log_method_call
    def delete_bgp_peer(self, context, bgp_peer_id):
        with db_api.context_manager.writer.using(context):
            super(MidonetBgpPlugin, self).delete_bgp_peer(
                context, bgp_peer_id)
            self.client.delete_bgp_peer_precommit(context, bgp_peer_id)

        self.client.delete_bgp_peer_postcommit(bgp_peer_id)

    @log_helpers.log_method_call
    def remove_bgp_peer(self, context, bgp_speaker_id, bgp_peer_info):
        with db_api.context_manager.writer.using(context):
            bgp_peer = super(MidonetBgpPlugin, self).remove_bgp_peer(
                context, bgp_speaker_id, bgp_peer_info)
            bgp_peer_id = bgp_peer_info['bgp_peer_id']
            self.client.delete_bgp_peer_precommit(context, bgp_peer_id)

        self.client.delete_bgp_peer_postcommit(bgp_peer_id)
        return bgp_peer

    @log_helpers.log_method_call
    def update_bgp_peer(self, context, bgp_peer_id, bgp_peer):
        with db_api.context_manager.writer.using(context):
            bgp_peer = super(MidonetBgpPlugin, self).update_bgp_peer(
                context, bgp_peer_id, bgp_peer)
            updated_bgp_peer = {'bgp_peer': bgp_peer}
            self.client.update_bgp_peer_precommit(
                context, bgp_peer_id, updated_bgp_peer)

        self.client.update_bgp_peer_postcommit(bgp_peer_id,
                                               updated_bgp_peer)
        return bgp_peer

    @log_helpers.log_method_call
    def get_advertised_routes(self, context, bgp_speaker_id):
        rt_id = self.get_router_associated_with_bgp_speaker(
            context, bgp_speaker_id)

        # figure out advertised routes from attached networks
        attached_routes = self.get_routes_from_attached_networks(
            context, bgp_speaker_id, rt_id)

        # prepare nexthops to change nexthop of extra
        # route to IP address of router port
        nexthops = list(set([route[1] for route in attached_routes]))

        # figure out advertised routes from extra routes
        extra_routes = self.get_routes_from_extra_routes(
            context, rt_id, nexthops)

        routes = attached_routes + extra_routes
        info = ({'destination': x, 'next_hop': y} for x, y in routes)
        return self._make_advertised_routes_dict(itertools.chain(info))

    @log_helpers.log_method_call
    def add_gateway_network(self, context, bgp_speaker_id, network_info):
        with db_api.context_manager.writer.using(context):
            # TODO(kengo): This validation is temporary workaround
            # until upstream adds a validation for
            # existing combination of bgp speaker and gateway network.
            if self.get_router_associated_with_bgp_speaker(
                    context, bgp_speaker_id):
                raise bsri.BgpSpeakerInUse(
                    id=bgp_speaker_id,
                    reason='is already associated with router.')
            core_plugin = directory.get_plugin()
            if not core_plugin._network_is_external(
                    context, network_info['network_id']):
                raise bsri.NetworkTypeInvalid()
            info = super(MidonetBgpPlugin, self).add_gateway_network(
                context, bgp_speaker_id, network_info)
            self.set_router_for_bgp_speaker_by_network(
                context, bgp_speaker_id, network_info['network_id'])

        return info

    @log_helpers.log_method_call
    def remove_gateway_network(self, context, bgp_speaker_id, network_info):
        with db_api.context_manager.writer.using(context):
            if self.get_bgp_peers_by_bgp_speaker(context, bgp_speaker_id):
                raise bsri.BgpSpeakerInUse(id=bgp_speaker_id)
            info = super(MidonetBgpPlugin, self).remove_gateway_network(
                context, bgp_speaker_id, network_info)
            self.delete_bgp_speaker_router_insertion(
                context, bgp_speaker_id)

        return info



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from oslo_utils import uuidutils

from neutron.agent.linux import utils
from neutron.conf.agent import common as config
from neutron.tests.unit.agent.linux import test_interface as n_test

from midonet.neutron.agent import interface


class TestMidonetInterfaceDriver(n_test.TestBase):
    def setUp(self):
        self.conf = config.setup_conf()
        config.register_interface_opts(self.conf)
        self.driver = interface.MidonetInterfaceDriver(self.conf)
        self.network_id = uuidutils.generate_uuid()
        self.port_id = uuidutils.generate_uuid()
        self.device_name = "tap0"
        self.mac_address = "aa:bb:cc:dd:ee:ff"
        self.bridge = "br-test"
        self.namespace = "ns-test"
        super(TestMidonetInterfaceDriver, self).setUp()

    def test_plug(self):
        cmd = ['mm-ctl', '--bind-port', self.port_id, 'tap0']
        self.device_exists.return_value = False

        root_dev = mock.Mock()
        ns_dev = mock.Mock()
        self.ip().add_veth = mock.Mock(return_value=(root_dev, ns_dev))
        with mock.patch.object(utils, 'execute') as execute:
            self.driver.plug(
                self.network_id, self.port_id,
                self.device_name, self.mac_address,
                self.bridge, self.namespace)
            execute.assert_called_once_with(cmd, run_as_root=True)

        expected = [mock.call(), mock.call(),
                    mock.call().add_veth(self.device_name,
                                         self.device_name,
                                         namespace2=self.namespace)]

        ns_dev.assert_has_calls(
            [mock.call.link.set_address(self.mac_address)])

        root_dev.assert_has_calls(
            [mock.call.disable_ipv6(), mock.call.link.set_up()])
        ns_dev.assert_has_calls([mock.call.link.set_up()])
        self.ip.assert_has_calls(expected, True)

    def test_unplug(self):
        self.driver.unplug(self.device_name, self.bridge, self.namespace)

        self.ip_dev.assert_has_calls([
            mock.call(self.device_name, namespace=self.namespace),
            mock.call().link.delete()])
        self.ip.assert_has_calls([mock.call().garbage_collect_namespace()])



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools

from neutron_lib.api.definitions import provider_net

from neutron.plugins.ml2 import driver_context as ctx

from midonet.neutron._i18n import _
from midonet.neutron.common import constants as const


def is_midonet_network(context):
    """Checks whether the context is mech driver context for MidoNet driver """

    if isinstance(context, ctx.NetworkContext):
        net = context.current
    elif isinstance(context, ctx.PortContext):
        net = context.network.current
    elif isinstance(context, ctx.SubnetContext):
        net = context.network.current
    else:
        raise ValueError(_("Invalid Mechanism driver context passed in."))

    return net.get(provider_net.NETWORK_TYPE) in const.MIDONET_NET_TYPES


def filter_midonet_network(func):
    """Decorator to filter out only the midonet network type"""
    @functools.wraps(func)
    def wrapper(self, context):
        if is_midonet_network(context):
            func(self, context)
    return wrapper



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants as n_const

from neutron_fwaas.tests.unit.services.firewall import test_fwaas_plugin as tfp

from midonet.neutron.tests.unit import test_midonet_plugin_ml2 as test_mn_ml2

# Overwrite the FWaaS plugin constant so that the MidoNet FWaaS plugin gets
# loaded when the tests run.
tfp.FW_PLUGIN_KLASS = "midonet_firewall"


class FirewallTestCaseMixin(object):

    def test_create_sets_status_to_active(self):
        with self.router() as router:
            router_ids = [router['router']['id']]
            with self.firewall(router_ids=router_ids) as firewall:
                # Verify that the firewall is ACTIVE
                fw_id = firewall['firewall']['id']
                req = self.new_show_request('firewalls', fw_id)
                res = self.deserialize(self.fmt,
                                       req.get_response(self.ext_api))
                status = res['firewall']['status']
                self.assertEqual(n_const.ACTIVE, status)

    def test_create_sets_status_to_inactive(self):
        with self.firewall(router_ids=[]) as firewall:
            # Verify that the firewall is INACTIVE
            fw_id = firewall['firewall']['id']
            req = self.new_show_request('firewalls', fw_id)
            res = self.deserialize(self.fmt,
                                   req.get_response(self.ext_api))
            status = res['firewall']['status']
            self.assertEqual(n_const.INACTIVE, status)

    def test_create_firewall_error_deletes_firewall(self):
        self.client_mock.create_firewall.side_effect = Exception("Fake Error")
        with self.router() as router:
            router_ids = [router['router']['id']]

            self._create_firewall(self.fmt, "fw_test", "fw_desc",
                                  router_ids=router_ids,
                                  expected_res_status=500)

            # Verify that the firewall was deleted from DB
            req = self.new_list_request('firewalls')
            res = self.deserialize(self.fmt,
                                   req.get_response(self.ext_api))
            self.assertFalse(res['firewalls'])

    def test_update_sets_status_to_active(self):
        with self.router() as router:
            router_ids = [router['router']['id']]
            with self.firewall(router_ids=router_ids) as firewall:

                fw_id = firewall['firewall']['id']
                data = {'firewall': {'name': 'foo'}}
                req = self.new_update_request('firewalls', data, fw_id)
                res = self.deserialize(self.fmt,
                                       req.get_response(self.ext_api))

                # Verify that the firewall is ACTIVE
                req = self.new_show_request('firewalls', fw_id)
                res = self.deserialize(self.fmt,
                                       req.get_response(self.ext_api))
                status = res['firewall']['status']
                self.assertEqual(n_const.ACTIVE, status)

    def test_update_sets_status_to_inactive(self):
        with self.router() as router:
            router_ids = [router['router']['id']]
            with self.firewall(router_ids=router_ids) as firewall:

                fw_id = firewall['firewall']['id']
                data = {'firewall': {'router_ids': []}}
                req = self.new_update_request('firewalls', data, fw_id)
                res = self.deserialize(self.fmt,
                                       req.get_response(self.ext_api))

                # Verify that the firewall is INACTIVE
                req = self.new_show_request('firewalls', fw_id)
                res = self.deserialize(self.fmt,
                                       req.get_response(self.ext_api))
                status = res['firewall']['status']
                self.assertEqual(n_const.INACTIVE, status)

    def test_update_firewall_error_sets_status_to_error(self):
        self.client_mock.update_firewall.side_effect = Exception("Fake Error")
        with self.router() as router:
            router_ids = [router['router']['id']]
            with self.firewall(router_ids=router_ids) as firewall:

                # Verify that an error while updating sets status to ERROR
                fw_id = firewall['firewall']['id']
                data = {'firewall': {'name': 'foo'}}
                req = self.new_update_request('firewalls', data, fw_id)
                res = req.get_response(self.ext_api)
                self.assertEqual(500, res.status_int)

                req = self.new_show_request('firewalls', fw_id)
                res = self.deserialize(self.fmt,
                                       req.get_response(self.ext_api))
                status = res['firewall']['status']
                self.assertEqual(n_const.ERROR, status)

    def test_delete_error_in_midonet_does_not_delete_firewall(self):
        self.client_mock.delete_firewall.side_effect = Exception("Fake Error")
        with self.router() as router:
            router_ids = [router['router']['id']]
            with self.firewall(router_ids=router_ids,
                               do_delete=False) as firewall:

                # Verify that deletion error from midonet side does not end up
                # deleting the firewall in DB
                fw_id = firewall['firewall']['id']
                req = self.new_delete_request('firewalls', fw_id)
                res = req.get_response(self.ext_api)
                self.assertEqual(500, res.status_int)

                req = self.new_show_request('firewalls', fw_id)
                res = req.get_response(self.ext_api)
                self.assertEqual(200, res.status_int)

    # The following tests do not apply to midonet since the firewall resources
    # are not in PENDING_CREATE state upon creation.

    def test_update_firewall_fails_when_firewall_pending(self):
        pass

    def test_update_firewall_policy_fails_when_firewall_pending(self):
        pass

    def test_update_firewall_rule_fails_when_firewall_pending(self):
        pass

    def test_list_firewalls_with_filtering(self):
        pass


class FirewallTestCaseML2(FirewallTestCaseMixin,
                          tfp.TestFirewallPluginBase,
                          test_mn_ml2.MidonetPluginML2TestCase):
    pass



# Copyright (C) 2015 Midokura SARL.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import portbindings
from neutron_lib import constants as n_const
from neutron_lib import exceptions as n_exc
from neutron_lib.plugins.ml2 import api
from oslo_config import cfg
from oslo_log import helpers as log_helpers
from oslo_log import log as logging

from midonet.neutron._i18n import _
from midonet.neutron.client import base as c_base
from midonet.neutron.common import config  # noqa
from midonet.neutron.common import constants as const
from midonet.neutron.common import utils as c_utils
from midonet.neutron.db import v2_migration
from midonet.neutron.ml2 import sg_callback
from midonet.neutron.ml2 import util as m_util
from midonet.neutron.services.qos import driver as qos_driver

LOG = logging.getLogger(__name__)


class MidonetMechanismDriver(api.MechanismDriver):

    """ML2 Mechanism Driver for Midonet."""

    def __init__(self):
        v2_migration.migrate()
        self.vif_type = const.VIF_TYPE_MIDONET
        self.supported_vnic_types = [portbindings.VNIC_NORMAL]
        self.vif_details = {portbindings.CAP_PORT_FILTER: True}

        self.client = c_base.load_client(cfg.CONF.MIDONET)
        self.client.initialize()

        qos_driver.register()

    def initialize(self):
        self.sec_handler = sg_callback.MidonetSecurityGroupsHandler(
            self.client)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def create_network_precommit(self, context):
        network = context.current
        self.client.create_network_precommit(context, network)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def create_network_postcommit(self, context):
        network = context.current
        self.client.create_network_postcommit(network)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def update_network_precommit(self, context):
        net = context.current
        self.client.update_network_precommit(context, net['id'], net)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def update_network_postcommit(self, context):
        net = context.current
        self.client.update_network_postcommit(net['id'], net)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def delete_network_precommit(self, context):
        network_id = context.current['id']
        self.client.delete_network_precommit(context, network_id)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def delete_network_postcommit(self, context):
        network_id = context.current['id']
        self.client.delete_network_postcommit(network_id)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def create_subnet_precommit(self, context):
        subnet = context.current
        self.client.create_subnet_precommit(context, subnet)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def create_subnet_postcommit(self, context):
        subnet = context.current
        self.client.create_subnet_postcommit(subnet)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def update_subnet_precommit(self, context):
        subnet = context.current
        self.client.update_subnet_precommit(context, subnet['id'], subnet)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def update_subnet_postcommit(self, context):
        subnet = context.current
        self.client.update_subnet_postcommit(subnet['id'], subnet)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def delete_subnet_precommit(self, context):
        subnet_id = context.current['id']
        self.client.delete_subnet_precommit(context, subnet_id)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def delete_subnet_postcommit(self, context):
        subnet_id = context.current['id']
        self.client.delete_subnet_postcommit(subnet_id)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def create_port_precommit(self, context):
        port = context.current
        self.client.create_port_precommit(context, port)

    def _validate_port_create(self, port):
        if (port.get('device_owner') == n_const.DEVICE_OWNER_ROUTER_GW
                and not port['fixed_ips']):
            msg = (_("No IPs assigned to the gateway port for"
                     " router %s") % port['device_id'])
            raise n_exc.BadRequest(resource='router', msg=msg)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def create_port_postcommit(self, context):
        port = context.current
        self._validate_port_create(port)
        self.client.create_port_postcommit(port)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def update_port_precommit(self, context):
        port = context.current
        c_utils.check_update_port(context.original, port)
        self.client.update_port_precommit(context, port['id'], port)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def update_port_postcommit(self, context):
        port = context.current
        self.client.update_port_postcommit(port['id'], port)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def delete_port_precommit(self, context):
        port_id = context.current['id']
        self.client.delete_port_precommit(context, port_id)

    @m_util.filter_midonet_network
    @log_helpers.log_method_call
    def delete_port_postcommit(self, context):
        port_id = context.current['id']
        self.client.delete_port_postcommit(port_id)

    @log_helpers.log_method_call
    def bind_port(self, context):
        for segment in context.segments_to_bind:
            if segment['network_type'] in const.MIDONET_NET_TYPES:
                context.set_binding(segment[api.ID],
                                    self.vif_type,
                                    self.vif_details,
                                    n_const.PORT_STATUS_ACTIVE)
                break
            else:
                LOG.debug(('midonet mechanism driver did NOT bind '
                           'port for segment %r'), segment)



# Copyright 2016 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""bgp speaker router insertion

Revision ID: f8b289f2644f
Revises: 3df8cd6d2ada
Create Date: 2016-05-13 10:14:14.939657

"""

# revision identifiers, used by Alembic.
revision = 'f8b289f2644f'
down_revision = '3df8cd6d2ada'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'bgp_speaker_router_associations',
        sa.Column('bgp_speaker_id', sa.String(length=36), nullable=False),
        sa.Column('router_id', sa.String(length=36), nullable=False,
                  unique=True),
        sa.ForeignKeyConstraint(
            ['bgp_speaker_id'], ['bgp_speakers.id'], ondelete='CASCADE'),
        sa.ForeignKeyConstraint(
            ['router_id'], ['routers.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('bgp_speaker_id'))



# Copyright 2015 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""initial

Revision ID: 29cbb88b092
Revises: 422da2897701
Create Date: 2015-11-20 15:16:18.501828

"""

from neutron.db.migration import cli


# revision identifiers, used by Alembic.
revision = '29cbb88b092'
down_revision = '422da2897701'
branch_labels = (cli.EXPAND_BRANCH,)


def upgrade():
    pass



# Copyright (C) 2015 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import validators
from oslo_log import helpers as log_helpers
from oslo_log import log as logging
from oslo_utils import excutils

from networking_l2gw import extensions as l2gateway_ext
from networking_l2gw.services.l2gateway.common import l2gw_validators
from networking_l2gw.services.l2gateway import plugin as l2gw_plugin
from neutron.api import extensions as neutron_extensions

from midonet.neutron.common import constants as mido_const
from midonet.neutron.db import l2gateway_midonet as l2gw_db
from midonet.neutron.services.l2gateway.common import l2gw_midonet_validators

LOG = logging.getLogger(__name__)


class MidonetL2GatewayPlugin(l2gw_plugin.L2GatewayPlugin,
                             l2gw_db.MidonetL2GatewayMixin):
    """Implementation of the Neutron l2 gateway Service Plugin.

    This class manages the workflow of Midonet l2 Gateway request/response.
    The base plugin methods are overridden because the MidoNet driver requires
    specific ordering of events.  For creation, the Neutron data must be
    created first, with the resource UUID generated.  Also, for both creation
    and deletion, by invoking the Neutron DB methods first, all the
    validations, such as 'check_admin()' are executed prior to attempting to
    modify the MidoNet data, preventing potential data inconsistency.
    """

    def __init__(self):
        # Dynamically change the validators so that they are applicable to
        # the MidoNet implementation of L2GW.
        # REVISIT(yamamoto): These validator modifications should not
        # have been here in the first place.  We should either put them
        # in upstream or remove them.
        l2gw_validators.validate_gwdevice_list = (l2gw_midonet_validators.
                                                  validate_gwdevice_list)
        val_type = validators._to_validation_type('l2gwdevice_list')
        validators.validators.pop(val_type, None)
        validators.add_validator(
            val_type,
            l2gw_midonet_validators.validate_gwdevice_list)
        l2gw_validators.validate_network_mapping_list = (
            l2gw_midonet_validators.
            validate_network_mapping_list_without_seg_id_validation)
        neutron_extensions.append_api_extensions_path(l2gateway_ext.__path__)
        super(MidonetL2GatewayPlugin, self).__init__()

    def add_port_mac(self, context, port_dict):
        # This function is not implemented now in MidoNet plugin.
        # We block this function in plugin level to prevent from loading
        # l2gw driver in upstream.
        self._get_driver_for_provider(mido_const.MIDONET_L2GW_PROVIDER
                                      ).add_port_mac(context, port_dict)

    def delete_port_mac(self, context, port):
        # This function is not implemented now in MidoNet plugin.
        # We block this function in plugin level to prevent from loading
        # l2gw driver in upstream.
        self._get_driver_for_provider(mido_const.MIDONET_L2GW_PROVIDER
                                      ).delete_port_mac(context, port)

    def create_l2_gateway(self, context, l2_gateway):
        # Gateway Device Management Service must be enabled
        # when Midonet L2 Gateway is used.
        self._check_and_get_gw_dev_service()
        self.validate_l2_gateway_for_create(context, l2_gateway)

        return l2gw_db.MidonetL2GatewayMixin.create_l2_gateway(
            self, context, l2_gateway)

    @log_helpers.log_method_call
    def create_l2_gateway_connection(self, context, l2_gateway_connection):

        self.validate_l2_gateway_connection_for_create(
            context, l2_gateway_connection)
        l2_gw_conn = (l2gw_db.MidonetL2GatewayMixin.
                      create_l2_gateway_connection(
                          self, context, l2_gateway_connection))

        # Copy over the ID so that the MidoNet driver knows about it.  ID is
        # necessary for MidoNet to process its translation.
        gw_connection = l2_gateway_connection[self.connection_resource]
        gw_connection["id"] = l2_gw_conn["id"]

        try:
            self._get_driver_for_provider(mido_const.MIDONET_L2GW_PROVIDER
                                          ).create_l2_gateway_connection(
                context, l2_gateway_connection)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error("Failed to create a l2 gateway connection "
                          "%(gw_conn_id)s in Midonet:%(err)s",
                          {"gw_conn_id": l2_gw_conn["id"], "err": ex})
                try:
                    l2gw_db.MidonetL2GatewayMixin.delete_l2_gateway_connection(
                        self, context, l2_gw_conn["id"])
                except Exception:
                    LOG.exception("Failed to delete a l2 gateway conn %s",
                                  l2_gw_conn["id"])
        return l2_gw_conn

    @log_helpers.log_method_call
    def delete_l2_gateway_connection(self, context, l2_gateway_connection):
        l2gw_db.MidonetL2GatewayMixin.delete_l2_gateway_connection(
            self, context, l2_gateway_connection)
        self._get_driver_for_provider(mido_const.MIDONET_L2GW_PROVIDER
                                      ).delete_l2_gateway_connection(
            context, l2_gateway_connection)



# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from neutronclient import shell

from midonet.neutron.extensions import logging_resource as log_res_ext
from midonet.neutron.tests.unit.neutronclient_ext import test_cli20
from midonet.neutron.tests.unit import test_extension_logging_resource as telr
from midonet.neutronclient.logging_resource_extension import _logging_resource

RESOURCE = 'logging_resource'
RESOURCES = 'logging_resources'
FAKE_TENANT_NAME = 'my_tenant_name'
FAKE_LOGGING_RESOURCE_ID = 'my_logging_resource_id'
FAKE_FIREWALL_LOG_ID1 = 'my_firewall_log_id1'
FAKE_FIREWALL_LOG_ID2 = 'my_firewall_log_id2'
FAKE_FIREWALL_ID1 = 'my_firewall_id1'
FAKE_FIREWALL_ID2 = 'my_firewall_id2'


class CLITestV20LoggingResourceJSON(test_cli20.CLIExtTestV20Base):

    def setUp(self):
        log_res = ("logging_resource", _logging_resource)
        self._mock_load_extensions(log_res)
        super(CLITestV20LoggingResourceJSON,
              self).setUp(plurals={'logging_resources': 'logging_resource'})
        self.register_non_admin_status_resource('logging_resource')

    def test_logging_resource_cmd_loaded(self):
        neutron_shell = shell.NeutronShell('2.0')
        log_res_cmd = {'logging-list':
                       _logging_resource.LoggingResourceList,
                       'logging-create':
                       _logging_resource.LoggingResourceCreate,
                       'logging-update':
                       _logging_resource.LoggingResourceUpdate,
                       'logging-delete':
                       _logging_resource.LoggingResourceDelete,
                       'logging-show':
                       _logging_resource.LoggingResourceShow
                       }
        for cmd_name, cmd_class in log_res_cmd.items():
            found = neutron_shell.command_manager.find_command([cmd_name])
            self.assertEqual(cmd_class, found[0])

    def _create_logging_resource(self, name, args,
                                 position_names, position_values):
        cmd = _logging_resource.LoggingResourceCreate(
            test_cli20.MyApp(sys.stdout), None)
        self._test_create_resource(RESOURCE, cmd, name,
                                   FAKE_LOGGING_RESOURCE_ID,
                                   args, position_names, position_values)

    def _update_logging_resource(self, args, values):
        cmd = _logging_resource.LoggingResourceUpdate(
            test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(RESOURCE, cmd, FAKE_LOGGING_RESOURCE_ID,
                                   args, values)

    def test_create_logging_resource(self):
        args = [telr.FAKE_LOG_RES_NAME,
                '--tenant_id', FAKE_TENANT_NAME,
                '--enabled', str(telr.ENABLED_TRUE),
                '--description', telr.FAKE_LOG_RES_DESC]
        position_names = ['name', 'tenant_id', 'enabled', 'description']
        position_values = [telr.FAKE_LOG_RES_NAME, FAKE_TENANT_NAME,
                           str(telr.ENABLED_TRUE), telr.FAKE_LOG_RES_DESC]
        self._create_logging_resource(telr.FAKE_LOG_RES_NAME, args,
                                      position_names, position_values)

    def test_update_logging_resource_with_name(self):
        args = [FAKE_LOGGING_RESOURCE_ID, '--name', telr.NEW_LOG_RES_NAME]
        values = {'name': telr.NEW_LOG_RES_NAME}
        self._update_logging_resource(args, values)

    def test_update_logging_resource_with_enabled(self):
        args = [FAKE_LOGGING_RESOURCE_ID, '--enabled', str(telr.ENABLED_FALSE)]
        values = {'enabled': str(telr.ENABLED_FALSE)}
        self._update_logging_resource(args, values)

    def test_delete_logging_resource(self):
        cmd = _logging_resource.LoggingResourceDelete(
            test_cli20.MyApp(sys.stdout), None)
        args = [FAKE_LOGGING_RESOURCE_ID]
        self._test_delete_resource(RESOURCE, cmd,
                                   FAKE_LOGGING_RESOURCE_ID, args)

    def test_list_logging_resources(self):
        cmd = _logging_resource.LoggingResourceList(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(RESOURCES, cmd)

    def test_list_logging_resources_with_pagination(self):
        cmd = _logging_resource.LoggingResourceList(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(RESOURCES, cmd)

    def test_list_logging_resource_with_firewall_logs(self):
        cmd = _logging_resource.LoggingResourceList(
            test_cli20.MyApp(sys.stdout), None)
        fw_log = [{"firewall_id": FAKE_FIREWALL_ID1,
                   "description": telr.FAKE_FW_LOG_DESC,
                   "id": FAKE_FIREWALL_LOG_ID1,
                   "fw_event": log_res_ext.FW_EVENT_ALL},
                  {"firewall_id": FAKE_FIREWALL_ID2,
                   "description": telr.FAKE_FW_LOG_DESC,
                   "id": FAKE_FIREWALL_LOG_ID2,
                   "fw_event": log_res_ext.FW_EVENT_DROP}]
        response = {'logging_resources': [
            {"id": FAKE_LOGGING_RESOURCE_ID,
             "name": telr.FAKE_LOG_RES_NAME,
             "enabled": str(telr.ENABLED_TRUE),
             "description": telr.FAKE_LOG_RES_DESC,
             "firewall_logs": fw_log}]}
        args = ['-c', 'id', '-c', 'firewall_logs']
        self._test_list_columns(cmd, RESOURCES, response, args)

    def test_show_logging_resource(self):
        cmd = _logging_resource.LoggingResourceShow(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(RESOURCE, cmd, self.test_id, args,
                                 ['id', 'name'])



# Copyright 2014 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add task

Revision ID: 25aeae45d4ad
Revises: None
Create Date: 2014-10-27 13:26:15.053541

"""

# revision identifiers, used by Alembic.
revision = '25aeae45d4ad'
down_revision = 'start_neutron_midonet'

from alembic import op
import sqlalchemy as sa


def upgrade():

    if op.get_bind().engine.dialect.name == 'mysql':
        # NOTE(yamamoto): Specify a length long enough to make
        # the dialict to choose LONGTEXT
        data_type = sa.Text(length=2 ** 24)
    else:
        data_type = sa.Text()

    op.create_table(
        'midonet_tasks',
        sa.Column('id', sa.Integer(), primary_key=True),
        sa.Column('type', sa.String(length=36)),
        sa.Column('data_type', sa.String(length=36)),
        sa.Column('data', data_type),
        sa.Column('resource_id', sa.String(length=36)),
        sa.Column('tenant_id', sa.String(length=255)),
        sa.Column('transaction_id', sa.String(length=40), nullable=False),
        sa.Column('created_at', sa.DateTime(), nullable=False),)



# Copyright (C) 2017 Midokura SARL.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from flake8_import_order import styles


class OpenStack(styles.PEP8):
    # We use application-package-names for OpenStack packages which
    # are not libraries.
    # See [flake8] section in tox.ini.
    # https://github.com/PyCQA/flake8-import-order#extending-styles
    accepts_application_package_names = True



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from midonetclient import client
from oslo_log import log as logging

from midonet.neutron.client import base
from midonet.neutron.common import utils

LOG = logging.getLogger(__name__)


class MidonetApiClient(base.MidonetClientBase):

    def __init__(self, conf):
        self.api_cli = client.MidonetClient(conf.midonet_uri, conf.username,
                                            conf.password,
                                            project_id=conf.project_id)
        self.config = conf

    def create_network_postcommit(self, network):
        self.api_cli.create_network(network)

    def update_network_postcommit(self, network_id, network):
        self.api_cli.update_network(network_id, network)

    def delete_network_postcommit(self, network_id):
        self.api_cli.delete_network(network_id)

    def create_subnet_postcommit(self, subnet):
        self.api_cli.create_subnet(subnet)

    def update_subnet_postcommit(self, subnet_id, subnet):
        self.api_cli.update_subnet(subnet_id, subnet)

    def delete_subnet_postcommit(self, subnet_id):
        self.api_cli.delete_subnet(subnet_id)

    def create_port_postcommit(self, port):
        self.api_cli.create_port(port)

    def update_port_postcommit(self, port_id, port):
        self.api_cli.update_port(port_id, port)

    def delete_port_postcommit(self, port_id):
        self.api_cli.delete_port(port_id)

    def create_router_postcommit(self, router):
        self.api_cli.create_router(router)

    def update_router_postcommit(self, router_id, router):
        self.api_cli.update_router(router_id, router)

    def delete_router_postcommit(self, router_id):
        self.api_cli.delete_router(router_id)

    def add_router_interface_postcommit(self, router_id, interface_info):
        self.api_cli.add_router_interface(router_id, interface_info)

    def remove_router_interface_postcommit(self, router_id, interface_info):
        self.api_cli.remove_router_interface(router_id, interface_info)

    def create_floatingip_postcommit(self, floatingip):
        self.api_cli.create_floating_ip(floatingip)

    def update_floatingip_postcommit(self, floatingip_id, floatingip):
        self.api_cli.update_floating_ip(floatingip_id, floatingip)

    def delete_floatingip_postcommit(self, floatingip_id):
        self.api_cli.delete_floating_ip(floatingip_id)

    def create_security_group_postcommit(self, security_group):
        self.api_cli.create_security_group(security_group)

    def delete_security_group_postcommit(self, security_group_id):
        self.api_cli.delete_security_group(security_group_id)

    def create_security_group_rule_postcommit(self, security_group_rule):
        self.api_cli.create_security_group_rule(security_group_rule)

    def create_security_group_rule_bulk_postcommit(self, security_group_rules):
        self.api_cli.create_security_group_rule_bulk(security_group_rules)

    def delete_security_group_rule_postcommit(self, security_group_rule_id):
        self.api_cli.delete_security_group_rule(security_group_rule_id)

    def create_vip(self, context, vip):
        self.api_cli.create_vip(vip)

    def update_vip(self, context, vip_id, vip):
        self.api_cli.update_vip(vip_id, vip)

    def delete_vip(self, context, vip_id):
        self.api_cli.delete_vip(vip_id)

    def create_pool(self, context, pool):
        self.api_cli.create_pool(pool)

    def update_pool(self, context, pool_id, pool):
        self.api_cli.update_pool(pool_id, pool)

    def delete_pool(self, context, pool_id):
        self.api_cli.delete_pool(pool_id)

    def create_member(self, context, member):
        self.api_cli.create_member(member)

    def update_member(self, context, member_id, member):
        self.api_cli.update_member(member_id, member)

    def delete_member(self, context, member_id):
        self.api_cli.delete_member(member_id)

    def create_health_monitor(self, context, health_monitor):
        self.api_cli.create_health_monitor(health_monitor)

    def update_health_monitor(self, context, health_monitor_id,
                              health_monitor):
        self.api_cli.update_health_monitor(health_monitor_id, health_monitor)

    def delete_health_monitor(self, context, health_monitor_id):
        self.api_cli.delete_health_monitor(health_monitor_id)

    def create_firewall(self, context, firewall):
        self.api_cli.create_firewall(firewall)

    def delete_firewall(self, context, firewall):
        self.api_cli.delete_firewall(firewall['id'])

    def update_firewall(self, context, firewall):
        self.api_cli.update_firewall(firewall['id'], firewall)

    def create_vpn_service(self, context, vpn_service):
        self.api_cli.create_vpn_service(vpn_service)

    def update_vpn_service(self, context, vpn_service_id, vpn_service):
        self.api_cli.update_vpn_service(vpn_service_id, vpn_service)

    def delete_vpn_service(self, context, vpn_service_id):
        self.api_cli.delete_vpn_service(vpn_service_id)

    def create_ipsec_site_conn(self, context, ipsec_site_conn):
        self.api_cli.create_ipsec_site_conn(ipsec_site_conn)

    def update_ipsec_site_conn(
            self, context, ipsec_site_conn_id, ipsec_site_conn):
        self.api_cli.update_ipsec_site_conn(
            ipsec_site_conn_id, ipsec_site_conn)

    def delete_ipsec_site_conn(self, context, ipsec_site_conn_id):
        self.api_cli.delete_ipsec_site_conn(ipsec_site_conn_id)

    def create_gateway_device_postcommit(self, gw_dev):
        self.api_cli.create_gateway_device(gw_dev)

    def update_gateway_device_postcommit(self, gw_dev_id, gw_dev):
        self.api_cli.update_gateway_device(gw_dev_id, gw_dev)

    def delete_gateway_device_postcommit(self, gw_dev_id):
        self.api_cli.delete_gateway_device(gw_dev_id)

    def create_gateway_device_remote_mac_entry_postcommit(self, mac_entry):
        self.api_cli.create_remote_mac_entry(mac_entry)

    def delete_gateway_device_remote_mac_entry_postcommit(self, mac_entry_id):
        self.api_cli.delete_remote_mac_entry(mac_entry_id)

    def create_l2_gateway_connection(self, context, l2_gw_conn):
        self.api_cli.create_l2gw_conn(l2_gw_conn)

    def delete_l2_gateway_connection(self, context, l2_gw_conn_id):
        self.api_cli.delete_l2gw_conn(l2_gw_conn_id)

    def update_bgp_speaker_postcommit(self, bgp_speaker_id, bgp_speaker):
        self.api_cli.update_bgp_speaker(bgp_speaker_id, bgp_speaker)

    def create_bgp_peer_postcommit(self, bgp_peer):
        self.api_cli.create_bgp_peer(bgp_peer)

    def update_bgp_peer_postcommit(self, bgp_peer_id, bgp_peer):
        self.api_cli.update_bgp_peer(bgp_peer_id, bgp_peer)

    def delete_bgp_peer_postcommit(self, bgp_peer_id):
        self.api_cli.delete_bgp_peer(bgp_peer_id)

    def update_logging_resource_postcommit(
            self, logging_resource_id, logging_resource):
        self.api_cli.update_logging_resource(
            logging_resource_id, logging_resource)

    def delete_logging_resource_postcommit(self, logging_resource_id):
        self.api_cli.delete_logging_resource(logging_resource_id)

    def create_firewall_log_postcommit(self, firewall_log):
        self.api_cli.create_firewall_log(firewall_log)

    def update_firewall_log_postcommit(self, firewall_log_id, firewall_log):
        self.api_cli.update_firewall_log(firewall_log_id, firewall_log)

    def delete_firewall_log_postcommit(self, firewall_log_id):
        self.api_cli.delete_firewall_log(firewall_log_id)

    def create_tap_flow(self, context, tap_flow):
        self.api_cli.create_tap_flow(tap_flow)

    def delete_tap_flow(self, context, tap_flow_id):
        self.api_cli.delete_tap_flow(tap_flow_id)

    def create_tap_service(self, context, tap_service):
        self.api_cli.create_tap_service(tap_service)

    def delete_tap_service(self, context, tap_service_id):
        self.api_cli.delete_tap_service(tap_service_id)

    # QoS
    def create_qos_policy(self, context, policy):
        policy_dict = policy.to_dict()
        self.api_cli.create_qos_policy(policy_dict)

    def update_qos_policy(self, context, policy):
        policy_dict = policy.to_dict()
        id_ = policy_dict['id']
        self.api_cli.update_qos_policy(id_, policy_dict)

    def delete_qos_policy(self, context, policy):
        policy_dict = policy.to_dict()
        id_ = policy_dict['id']
        self.api_cli.delete_qos_policy(id_)

    # MidoNet Pool Member
    def get_pool_member(self, context, member_id):
        return self.api_cli.get_pool_member(member_id)


def _build_func(method, lb_op):
    # update takes (context, old_obj, obj, **kwargs)
    # others take (context, obj, **kwargs)
    def f(self, context, **kwargs):
        obj_dict = kwargs['obj'].to_api_dict()
        id_ = obj_dict['id']
        try:
            api_method = getattr(self.api_cli, method)
        except AttributeError:
            # REVISIT(yamamoto):
            # not implemented.  this block can be removed once
            # these ops are implemented in python-midonetclient.
            LOG.debug('%(method)s is not implemented. obj = %(obj_dict)s', {
                'method': method,
                'obj_dict': obj_dict,
            })
            return
        if lb_op == 'create':
            return api_method(obj_dict)
        elif lb_op == 'update':
            return api_method(id_, obj_dict)
        else:  # delete, refresh, stats
            return api_method(id_)
    f.__name__ = method
    return f


# Add LBaaS v2 methods
for resource in [
    'loadbalancerv2',
    'listenerv2',
    'poolv2',
    'memberv2',
    'healthmonitorv2',
]:
    ops = ['create', 'update', 'delete']
    if resource == 'loadbalancerv2':
        ops += ['refresh', 'stats']
    for op in ops:
        method = op + '_' + resource
        f = _build_func(method, op)
        unbound = utils.unboundmethod(f, MidonetApiClient)
        setattr(MidonetApiClient, method, unbound)



# Copyright (C) 2015 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from neutron_lib.api import extensions as api_extensions
from neutron_lib.api import validators
from neutron_lib.db import constants as db_const
from neutron_lib import exceptions as nexception
from neutron_lib.plugins import directory
from oslo_log import log as logging
import six

from neutron.api import extensions
from neutron.api.v2 import base
from neutron.api.v2 import resource_helper

from midonet.neutron._i18n import _
from midonet.neutron.common import constants


class GatewayDeviceNotFound(nexception.NotFound):
    message = _("Gateway device %(id)s does not exist")


class RemoteMacEntryNotFound(nexception.NotFound):
    message = _("Remote MAC entry %(id)s does not exist")


class RemoteMacEntryWrongGatewayDevice(nexception.InvalidInput):
    message = _("Remote MAC entry %(id)s does not belong to gateway "
                "device %(gateway_device_id)s")


class ResourceNotFound(nexception.NotFound):
    message = _("specified resource %(resource_id)s does not exist")


class HwVtepTypeInvalid(nexception.InvalidInput):
    message = _("Gateway device %(type)s must be specified with "
                "management_port and management_ip")


class RouterVtepTypeInvalid(nexception.InvalidInput):
    message = _("Gateway device %(type)s must be specified with "
                "resource_id")


class NetworkVlanTypeInvalid(nexception.InvalidInput):
    message = _("Gateway device %(type)s must be specified with "
                "resource_id")


class DuplicateRemoteMacEntry(nexception.InUse):
    message = _("Request contains duplicate remote mac address entry: "
                "mac_address %(mac_address)s.")


class GatewayDeviceParamDuplicate(nexception.InUse):
    message = _("%(param_name)s %(param_value)s %(reason)s")

    def __init__(self, **kwargs):
        if 'reason' not in kwargs:
            kwargs['reason'] = "is already used"
        super(GatewayDeviceParamDuplicate, self).__init__(**kwargs)


class GatewayDeviceInUse(nexception.InUse):
    message = _("Gateway device %(id)s %(reason)s")

    def __init__(self, **kwargs):
        if 'reason' not in kwargs:
            kwargs['reason'] = "is in use by l2 gateway"
        super(GatewayDeviceInUse, self).__init__(**kwargs)


class DeviceInUseByGatewayDevice(nexception.InUse):
    message = _("device %(resource_id)s (%(resource_type)s) %(reason)s")

    def __init__(self, **kwargs):
        if 'reason' not in kwargs:
            kwargs['reason'] = "is in use by gateway device"
        super(DeviceInUseByGatewayDevice, self).__init__(**kwargs)


class TunnelIPsExhausted(nexception.BadRequest):
    message = _("Unable to complete operation for Gateway Device. "
                "The number of tunnel ips exceeds the maximum 1.")


class TunnelIPsRequired(nexception.BadRequest):
    message = _("Unable to complete operation for Gateway Device. "
                "The tunnel ips are required for %(gw_type)s type.")


class OperationRemoteMacEntryNotSupported(nexception.Conflict):
    message = _("Unable to operate remote_mac_entry for gateway device "
                "%(type)s type.")


def _validate_port_or_none(data, valid_values=None):
    if data is None:
        return
    return validators.validate_range(data, [0, 65535])


validators.add_validator('_midonet_port_or_none', _validate_port_or_none)


GATEWAY_DEVICE = 'gateway_device'
GATEWAY_DEVICES = '%ss' % GATEWAY_DEVICE

HW_VTEP_TYPE = 'hw_vtep'
ROUTER_DEVICE_TYPE = 'router_vtep'
NETWORK_VLAN_TYPE = 'network_vlan'
gateway_device_valid_types = [HW_VTEP_TYPE, ROUTER_DEVICE_TYPE,
                              NETWORK_VLAN_TYPE]

OVSDB = 'ovsdb'
gateway_device_valid_protocols = [OVSDB]

GATEWAY_DEVICE_PREFIX = '/gw'
PLURAL_IES = 'ies'

LOG = logging.getLogger(__name__)

# Attribute Map
RESOURCE_ATTRIBUTE_MAP = {
    'gateway_devices': {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'primary_key': True},
        'name': {'allow_post': True, 'allow_put': True,
                 'validate': {'type:string': db_const.NAME_FIELD_SIZE},
                 'default': "",
                 'is_visible': True},
        'type': {'allow_post': True, 'allow_put': False,
                 'validate': {'type:values': gateway_device_valid_types},
                 'default': HW_VTEP_TYPE,
                 'is_visible': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'validate': {
                          'type:string': db_const.PROJECT_ID_FIELD_SIZE
                      },
                      'is_visible': True},
        'management_ip': {'allow_post': True, 'allow_put': False,
                          'default': None,
                          'validate': {'type:ip_address_or_none': None},
                          'is_visible': True},
        'management_port': {'allow_post': True, 'allow_put': False,
                            'validate': {'type:_midonet_port_or_none': None},
                            'default': None, 'is_visible': True},
        'management_protocol': {'allow_post': True, 'allow_put': False,
                                'is_visible': True, 'default': None},
        'resource_id': {'allow_post': True, 'allow_put': False,
                        'validate': {
                            'type:string': db_const.DEVICE_ID_FIELD_SIZE},
                        'is_visible': True, 'required_by_policy': True,
                        'default': ""},
        'tunnel_ips': {'allow_post': True, 'allow_put': True,
                       'is_visible': True, 'default': ''},
        'remote_mac_entries': {'allow_post': False, 'allow_put': False,
                               'default': None,
                               'is_visible': True}
    }
}

SUB_RESOURCE_ATTRIBUTE_MAP = {
    'remote_mac_entries': {
        'parent': {'collection_name': 'gateway_devices',
                   'member_name': 'gateway_device'},
        'parameters': {'id': {
            'allow_post': False, 'allow_put': False,
            'validate': {'type:uuid': None},
            'is_visible': True,
            'primary_key': True},
            'vtep_address': {
                'allow_post': True, 'allow_put': False,
                'is_visible': True, 'default': None,
                'validate': {'type:ip_address': None}},
            'mac_address': {
                'allow_post': True, 'allow_put': False,
                'is_visible': True,
                'validate': {'type:mac_address': None}},
            'segmentation_id': {
                'allow_post': True, 'allow_put': False,
                'is_visible': True,
                'validate': {'type:non_negative': None}},
            # FIXME(kengo): Workaround to avoid 400 error
            # when issue creation request without tenant_id.
            # We will address with one of following solution this
            # after discussion with neutron.
            # 1. delete this definition if neutron core is modified.
            # 2. add DB column and remain this definition
            #    if neutron core is not modified.
            'tenant_id': {
                'allow_post': True,
                'allow_put': False,
                'required_by_policy': True,
                'validate': {'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                'is_visible': False}
        }
    }
}


class Gateway_device(api_extensions.ExtensionDescriptor):
    """Gateway device extension."""

    @classmethod
    def get_name(cls):
        return "Midonet Gateway Device Extension"

    @classmethod
    def get_alias(cls):
        return "gateway-device"

    @classmethod
    def get_description(cls):
        return "The gateway device extension."

    @classmethod
    def get_updated(cls):
        return "2015-11-11T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """Returns Ext Resources."""

        plural_mappings = resource_helper.build_plural_mappings(
            {}, RESOURCE_ATTRIBUTE_MAP)

        resources = resource_helper.build_resource_info(
            plural_mappings,
            RESOURCE_ATTRIBUTE_MAP,
            constants.GATEWAY_DEVICE)
        plugin = directory.get_plugin(constants.GATEWAY_DEVICE)

        for collection_name in SUB_RESOURCE_ATTRIBUTE_MAP:
            # Special handling needed for sub-resources with 'y' ending
            # (e.g. proxies -> proxy)
            if collection_name[-3:] == PLURAL_IES:
                resource_name = collection_name[:-3] + 'y'
            else:
                resource_name = collection_name[:-1]
            parent = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get('parent')
            params = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get(
                'parameters')

            controller = base.create_resource(collection_name, resource_name,
                                              plugin, params,
                                              allow_bulk=True,
                                              parent=parent)

            resource = extensions.ResourceExtension(
                collection_name,
                controller, parent,
                path_prefix=GATEWAY_DEVICE_PREFIX,
                attr_map=params)
            resources.append(resource)

        return resources

    def get_extended_resources(self, version):
        if version == "2.0":
            return RESOURCE_ATTRIBUTE_MAP
        else:
            return {}


@six.add_metaclass(abc.ABCMeta)
class GwDevicePluginBase(object):

    path_prefix = GATEWAY_DEVICE_PREFIX

    @abc.abstractmethod
    def create_gateway_device(self, context, gw_dev):
        pass

    @abc.abstractmethod
    def update_gateway_device(self, context, id, gw_dev):
        pass

    @abc.abstractmethod
    def delete_gateway_device(self, context, id):
        pass

    @abc.abstractmethod
    def get_gateway_devices(self, context, filters=None, fields=None,
                            sorts=None, limit=None, marker=None,
                            page_reverse=False):
        pass

    @abc.abstractmethod
    def get_gateway_device(self, context, id, fields=None):
        pass

    @abc.abstractmethod
    def create_gateway_device_remote_mac_entry(self, context,
                                               gateway_device_id,
                                               remote_mac_entry):
        pass

    @abc.abstractmethod
    def delete_gateway_device_remote_mac_entry(self, context,
                                               id, gateway_device_id):
        pass

    @abc.abstractmethod
    def get_gateway_device_remote_mac_entries(self, context, gateway_device_id,
                                              filters=None, fields=None,
                                              sorts=None, limit=None,
                                              marker=None, page_reverse=False):
        pass

    @abc.abstractmethod
    def get_gateway_device_remote_mac_entry(self, context, id,
                                            gateway_device_id, fields=None):
        pass



# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import exceptions as nexception

from midonet.neutron._i18n import _


class MidonetL2GatewayUnavailable(nexception.ServiceUnavailable):
    message = _("Midonet L2 Gateway Service is unavailable "
                "because Gateway Device Management Service is disabled.")


class MidonetL2GatewayConnectionExists(nexception.InUse):
    message = _("L2 Gateway Connection related to "
                "specified L2 Gateway %(l2_gateway_id)s already exists")



# Copyright (C) 2016 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sqlalchemy as sa
from sqlalchemy import orm

from neutron_lib.db import model_base

LOGGING_RESOURCES = 'midonet_logging_resources'
FIREWALL_LOGS = 'midonet_firewall_logs'


class LoggingResource(model_base.BASEV2, model_base.HasProjectNoIndex):
    """Represents a logging resource."""

    __tablename__ = LOGGING_RESOURCES
    id = sa.Column(sa.String(36), primary_key=True)
    name = sa.Column(sa.String(255))
    description = sa.Column(sa.String(1024))
    enabled = sa.Column(sa.Boolean, nullable=False)


class FirewallLog(model_base.BASEV2, model_base.HasProjectNoIndex):
    """Represents a firewall log."""

    __tablename__ = FIREWALL_LOGS

    id = sa.Column(sa.String(36), primary_key=True)
    logging_resource_id = sa.Column(
        sa.String(36),
        sa.ForeignKey('midonet_logging_resources.id', ondelete="CASCADE"),
        nullable=False)
    description = sa.Column(sa.String(1024))
    fw_event = sa.Column(sa.String(length=255), nullable=False)
    firewall_id = sa.Column(
        sa.String(36),
        sa.ForeignKey('firewalls.id', ondelete="CASCADE"),
        nullable=False)
    logging_resource = orm.relationship(
        LoggingResource,
        backref=orm.backref('firewall_logs', cascade='delete', lazy='joined'),
        primaryjoin="LoggingResource.id==FirewallLog.logging_resource_id")



# Copyright 2015 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""add network binding table

Revision ID: 422da2897701
Revises: 421564f630b1
Create Date: 2015-06-05 03:23:00.000000

"""

from alembic import op
import sqlalchemy as sa

from neutron.db import migration

# revision identifiers, used by Alembic.
revision = '422da2897701'
down_revision = '421564f630b1'

# milestone identifier, used by neutron-db-manage
neutron_milestone = [migration.LIBERTY]


def upgrade():
    op.create_table(
        'midonet_network_bindings',
        sa.Column('network_id', sa.String(length=36),
                  sa.ForeignKey('networks.id'), nullable=False,
                  primary_key=True),
        sa.Column('network_type', sa.String(length=255), nullable=False))
